@@ -1,4 +1,3 @@
-/* $Id: ipfail.c,v 1.24.2.5 2004/09/11 06:36:30 msoffen Exp $ */
 /* ipfail: IP Failover plugin for Linux-HA
  *
  * Copyright (C) 2002-2004 Kevin Dwyer <kevin@pheared.net>
@@ -39,7 +38,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <portability.h>
+#include <lha_internal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -60,7 +59,9 @@
 #include <clplumbing/cl_malloc.h>
 #include <clplumbing/GSource.h>
 #include <clplumbing/Gmain_timeout.h>
+#include <clplumbing/coredumps.h>
 #include "ipfail.h"
+#include <clplumbing/cl_misc.h>
 
 /* ICK! global vars. */
 const char *node_name;	   /* The node we are connected to            */
@@ -79,12 +80,12 @@ main(int argc, char **argv)
 	ll_cluster_t *hb;
 	char pid[10];
 	char *bname, *parameter;
-	int apifd;
+	IPC_Channel *apiIPC;
 
 	cl_log_enable_stderr(TRUE);
 	
 	/* Get the name of the binary for logging purposes */
-	bname = ha_strdup(argv[0]);
+	bname = cl_strdup(argv[0]);
 	cl_log_set_entity(basename(bname));
 
 	cl_log_set_facility(DEFAULT_FACILITY);
@@ -101,6 +102,10 @@ main(int argc, char **argv)
 	open_api(hb);
 
 	node_stable = is_stable(hb);
+	if (node_stable == -1) {
+		cl_log(LOG_ERR, "No managed resources");
+		exit(100);
+	}
 
 	/* Obtain our local node name */
 	node_name = hb->llc_ops->get_mynodeid(hb);
@@ -120,17 +125,26 @@ main(int argc, char **argv)
 			       "incompatible legacy option.");
 			exit(100);
 		}
-
-		if (!strcmp(parameter, "on"))
-			auto_failback = 1;
-		else
-			auto_failback = 0;
+		
+		if(cl_str_to_boolean(parameter, &auto_failback) != HA_OK){
+			cl_log(LOG_ERR, " invalid auto_faiback value(%s)",
+			       parameter);
+			exit(100);
+		}
 
 		cl_log(LOG_DEBUG, "auto_failback -> %i (%s)", auto_failback,
 		       parameter);
 		cl_free(parameter);
-	} else
+	}else{
 		cl_log(LOG_ERR, "Couldn't get auto_failback setting.");
+	}
+	/* See if we should drop cores somewhere odd... */
+	parameter = hb->llc_ops->get_parameter(hb, KEY_COREROOTDIR);
+	if (parameter) {
+		cl_set_corerootdir(parameter);
+		cl_cdtocoredir();
+	}
+	cl_cdtocoredir();
 
 
 	set_callbacks(hb);
@@ -158,11 +172,12 @@ main(int argc, char **argv)
 	 */
 	mainloop = g_main_new(TRUE);
 	
-	apifd = hb->llc_ops->inputfd(hb);
-	
-	/* Watch the API's fd for input */
-	G_main_add_fd(G_PRIORITY_HIGH, apifd, FALSE, ipfail_dispatch, 
-		      (gpointer)hb, ipfail_dispatch_destroy);
+        apiIPC = hb->llc_ops->ipcchan(hb);
+
+	/* Watch the API IPC for input */
+        G_main_add_IPC_Channel(G_PRIORITY_HIGH, apiIPC, FALSE, 
+                               ipfail_dispatch, (gpointer)hb,
+                               ipfail_dispatch_destroy);
 
 	Gmain_timeout_add_full(G_PRIORITY_DEFAULT, 1000, 
 	                     ipfail_timeout_dispatch, (gpointer)hb, 
@@ -184,7 +199,12 @@ main(int argc, char **argv)
 int
 is_stable(ll_cluster_t *hb)
 {
-	if (!strcmp(hb->llc_ops->get_resources(hb), "transition"))
+	const char *resources = hb->llc_ops->get_resources(hb);
+	if (!resources)
+		/* Heartbeat is not providing resource management */
+	        return -1;
+	
+	if (!strcmp(resources, "transition"))
 		return 0;
 
 	return 1;
@@ -209,7 +229,7 @@ node_walk(ll_cluster_t *hb)
 		/* Look for our partner */
 		if (!strcmp("normal", hb->llc_ops->node_type(hb, node))
 		    && strcmp(node, node_name)) {
-			strcpy(other_node, node);
+			strlcpy(other_node, node, sizeof(other_node));
 			cl_log(LOG_DEBUG, "[They are %s]", other_node);
 		}
 
@@ -509,14 +529,14 @@ send_abort_giveup(ll_cluster_t *hb)
 }
 
 void
-msg_abort_giveup(const struct ha_msg *msg, void *private)
+msg_abort_giveup(struct ha_msg *msg, void *private)
 {
         abort_giveup();
 }
 
 
 void
-msg_ipfail_join(const struct ha_msg *msg, void *private)
+msg_ipfail_join(struct ha_msg *msg, void *private)
 {
 	/* msg_ipfail_join: When another ipfail client sends a join 
 	 * message, call ask_ping_nodes() to compare ping node counts.
@@ -535,7 +555,7 @@ msg_ipfail_join(const struct ha_msg *msg, void *private)
 }
 
 void
-msg_resources(const struct ha_msg *msg, void *private)
+msg_resources(struct ha_msg *msg, void *private)
 {
 	const char * orig = ha_msg_value(msg, F_ORIG);
 	const char * isstable = ha_msg_value(msg, F_ISSTABLE);
@@ -580,7 +600,7 @@ msg_resources(const struct ha_msg *msg, void *private)
 			 * ping nodes.
 			 */
 			ask_ping_nodes(private, ping_node_status(private));
-			/*giveup(private); */
+			/* giveup(private); */
 
                         /* The ask_ping_nodes message will sort out whether
                          * a standby is necessary. */
@@ -625,7 +645,7 @@ ask_ping_nodes(ll_cluster_t *hb, int num_ping)
 }
 
 void
-msg_ping_nodes(const struct ha_msg *msg, void *private)
+msg_ping_nodes(struct ha_msg *msg, void *private)
 {
 	/* msg_ping_nodes: Takes the message and heartbeat cluster as input;
 	 * returns nothing.  Callback for the num_ping_nodes message.
@@ -691,7 +711,7 @@ you_are_dead(ll_cluster_t *hb)
 }
 
 void
-i_am_dead(const struct ha_msg *msg, void *private)
+i_am_dead(struct ha_msg *msg, void *private)
 {
 	/* i_am_dead: Takes the you_are_dead message and the heartbeat cluster
 	 * as input; returns nothing.
@@ -712,7 +732,7 @@ gotsig(int nsig)
 
 /* Used to handle the API in the gmainloop */
 gboolean 
-ipfail_dispatch(int fd, gpointer user_data)
+ipfail_dispatch(IPC_Channel* ipc, gpointer user_data)
 {
 	struct ha_msg *reply;
 	ll_cluster_t *hb = user_data;
@@ -755,7 +775,7 @@ ipfail_timeout_dispatch(gpointer user_data)
 
 	if (hb->llc_ops->msgready(hb)) {
 		/* cl_log(LOG_DEBUG, "Msg ready! [2]"); */
-		return ipfail_dispatch(-1, user_data);
+		return ipfail_dispatch(NULL, user_data);
 	}
 	return TRUE;
 }
@@ -782,7 +802,7 @@ void
 close_api(ll_cluster_t *hb)
 {
 	/* Log off of the API and clean up */
-	if (hb->llc_ops->signoff(hb) != HA_OK) {
+	if (hb->llc_ops->signoff(hb, FALSE) != HA_OK) {
 		cl_log(LOG_ERR, "Cannot sign off from heartbeat.");
 		cl_log(LOG_ERR, "REASON: %s", hb->llc_ops->errmsg(hb));
 		exit(14);
