@@ -1,4 +1,3 @@
-/* $Id: hb_api.c,v 1.94.2.6 2005/11/14 20:55:29 gshi Exp $ */
 /*
  * hb_api: Server-side heartbeat API code
  *
@@ -7,19 +6,19 @@
  *
  * Thanks to Conectiva S.A. for sponsoring Marcelo Tosatti work
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- * 
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 /*
@@ -65,7 +64,7 @@
  *	same UID on each machine.
  */
 
-#include <portability.h>
+#include <lha_internal.h>
 #include <sys/time.h>
 #define	time FOOtime
 #include <glib.h>
@@ -87,6 +86,7 @@
 #include <unistd.h>
 #include <clplumbing/cl_poll.h>
 #include <clplumbing/cl_signal.h>
+#include <clplumbing/netstring.h>
 
 /* Definitions of API query handlers */
 static int api_ping_iflist(const struct ha_msg* msg, struct node_info * node
@@ -109,6 +109,12 @@ static int api_nodelist (const struct ha_msg* msg, struct ha_msg* resp
 static int api_nodestatus (const struct ha_msg* msg, struct ha_msg* resp
 ,	client_proc_t* client, const char** failreason);
 
+static int api_nodeweight (const struct ha_msg* msg, struct ha_msg* resp
+,	client_proc_t* client, const char** failreason);
+
+static int api_nodesite (const struct ha_msg* msg, struct ha_msg* resp
+,	client_proc_t* client, const char** failreason);
+
 static int api_nodetype (const struct ha_msg* msg, struct ha_msg* resp
 ,	client_proc_t* client, const char** failreason);
 
@@ -121,12 +127,25 @@ static int api_iflist (const struct ha_msg* msg, struct ha_msg* resp
 static int api_clientstatus (const struct ha_msg* msg, struct ha_msg* resp
 ,	client_proc_t* client, const char** failreason);
 
+static int
+api_num_nodes(const struct ha_msg* msg, struct ha_msg* resp
+	      ,	client_proc_t* client, const char** failreason);
+
 static int api_get_parameter (const struct ha_msg* msg, struct ha_msg* resp
 ,	client_proc_t* client, const char** failreason);
 
 static int api_get_resources (const struct ha_msg* msg, struct ha_msg* resp
 ,	client_proc_t* client, const char** failreason);
 
+static int api_get_uuid (const struct ha_msg* msg, struct ha_msg* resp
+,	client_proc_t* client, const char** failreason);
+
+static int api_get_nodename (const struct ha_msg* msg, struct ha_msg* resp
+,	client_proc_t* client, const char** failreason);
+static int
+api_set_sendqlen(const struct ha_msg* msg, struct ha_msg* resp,
+		 client_proc_t* client, const char** failreason);
+
 gboolean ProcessAnAPIRequest(client_proc_t* client);
 
 struct api_query_handler query_handler_list [] = {
@@ -135,12 +154,18 @@ struct api_query_handler query_handler_list [] = {
 	{ API_SETSIGNAL, api_setsignal },
 	{ API_NODELIST, api_nodelist },
 	{ API_NODESTATUS, api_nodestatus },
+	{ API_NODEWEIGHT, api_nodeweight },
+	{ API_NODESITE, api_nodesite },
 	{ API_NODETYPE, api_nodetype },
 	{ API_IFSTATUS, api_ifstatus },
 	{ API_IFLIST, api_iflist },
 	{ API_CLIENTSTATUS, api_clientstatus },
+	{ API_NUMNODES, api_num_nodes},
 	{ API_GETPARM, api_get_parameter},
 	{ API_GETRESOURCES, api_get_resources},
+	{ API_GETUUID, api_get_uuid},
+	{ API_GETNAME, api_get_nodename},
+	{ API_SET_SENDQLEN, api_set_sendqlen}	
 };
 
 extern int	UseOurOwnPoll;
@@ -150,6 +175,9 @@ client_proc_t*	client_list = NULL;	/* List of all our API clients */
 			/* TRUE when any client output still pending */
 extern struct node_info *curnode;
 
+static unsigned long	client_generation = 0; 
+#define MAX_CLIENT_GEN 64
+
 static void	api_process_request(client_proc_t* client, struct ha_msg *msg);
 static void	api_send_client_msg(client_proc_t* client, struct ha_msg *msg);
 static void	api_send_client_status(client_proc_t* client
@@ -160,9 +188,169 @@ static void	G_remove_client(gpointer Client);
 static gboolean	APIclients_input_dispatch(IPC_Channel* chan, gpointer udata);
 static void	api_process_registration_msg(client_proc_t*, struct ha_msg *);
 static gboolean	api_check_client_authorization(client_proc_t* client);
-
+static int	create_seq_snapshot_table(GHashTable** ptable) ;
+static void	destroy_seq_snapshot_table(GHashTable* table);
 extern GHashTable*	APIAuthorization;
 
+struct seq_snapshot{
+	seqno_t		generation;
+	seqno_t		last_seq;
+};
+
+
+
+static int
+should_msg_sendto_client(client_proc_t* client, struct ha_msg* msg)
+{
+	GHashTable* table;
+	struct node_info *	thisnode = NULL;	
+	const char *		from;
+	cl_uuid_t		fromuuid;
+	struct seq_snapshot*	snapshot;
+	const char *		cseq;
+	const char *		cgen;
+	seqno_t			seq;
+	seqno_t			gen;
+	int			ret = 0;
+	const char*		type;
+	struct seqtrack *	t;
+
+	if (!client || !msg){
+		cl_log(LOG_ERR, "should_msg_sendto_client:"
+		       " invalid arguemts");
+		return FALSE;
+	}
+	
+	
+
+	from = ha_msg_value(msg, F_ORIG);
+	cseq = ha_msg_value(msg, F_SEQ);
+	cgen = ha_msg_value(msg, F_HBGENERATION);
+	
+	if (!from || !cseq || !cgen){
+		/* some local generated status messages,
+		 * e.g. node dead status message,
+		 * return yes
+		 */
+		return TRUE;
+	}
+	
+	if (sscanf(cseq, "%lx", &seq) <= 0 || sscanf(cgen, "%lx", &gen) <= 0) {
+		cl_log(LOG_ERR, "should_msg_sendto_client:"
+		       "wrong seq/gen format");
+		return FALSE;
+	}
+	
+	if (seq < 0 || gen < 0){
+		cl_log(LOG_ERR, "should_msg_sendto_client:"
+		       "wrong seq/gen number");
+		return FALSE;
+	}
+	
+	cl_get_uuid(msg, F_ORIGUUID, &fromuuid);
+	thisnode = lookup_tables(from, &fromuuid);
+	if ( thisnode == NULL){
+		cl_log(LOG_ERR, "should_msg_sendto_client:"
+		       "node not found in table");
+		return FALSE;
+	}
+
+	t = &thisnode->track;
+
+	/*if uuid is not found, then it always passes the first restriction*/
+	if ( cl_uuid_is_null(&fromuuid)
+	     || (table = client->seq_snapshot_table)== NULL
+	     || (snapshot= (struct seq_snapshot*)
+		 g_hash_table_lookup(table, &fromuuid)) == NULL){
+		goto nextstep;
+	}
+		
+	ret = gen > snapshot->generation ||
+		(gen == snapshot->generation && seq >= snapshot->last_seq);
+	
+
+	/*check if there is any retransmission going on
+	  if not, we can delete this item
+	*/
+
+	if (t->nmissing == 0){
+		
+		if (ANYDEBUG){
+			cl_log(LOG_DEBUG,
+			       "Removing one entry in seq snapshot hash table"
+			       "for node %s", thisnode->nodename);
+		}
+		
+		if(!g_hash_table_remove(table, &fromuuid)){
+			cl_log(LOG_ERR,"should_msg_sendto_client:"
+			       "g_hash_table_remove failed");
+			return FALSE;
+		}
+		cl_free(snapshot);
+		
+		
+		if ( g_hash_table_size(table) ==0){
+			if (ANYDEBUG){
+				cl_log(LOG_DEBUG,
+				       "destroying the seq snapshot hash table");
+			}
+
+			g_hash_table_destroy(table);
+			client->seq_snapshot_table = NULL;
+		}
+	}		
+	
+	if ( ret == 0  ){
+		/* hmmmm.... this message is dropped */
+		cl_log(LOG_WARNING, "message is dropped ");
+		cl_log_message(LOG_WARNING, msg);
+		return FALSE;
+	}
+	
+ nextstep:
+        /* We only worry about the ordering of certain types of messages
+	 * and then only when they arrive out of order.
+	 * Basically we implement a barrier at the receipt of each
+	 * message of this type.
+	 */
+	if( (type = ha_msg_value(msg, F_TYPE)) == NULL){
+		cl_log(LOG_ERR, "no type field found");
+		return FALSE;
+	}
+	
+	if ( strcmp(type, T_APICLISTAT) != 0 || t->nmissing == 0){		
+		return TRUE;
+	}
+	
+
+	if ( seq > t->first_missing_seq ){
+		/*We cannot deliver the message now,
+		  queue it*/		
+		
+		struct ha_msg* copymsg = ha_msg_copy(msg);
+		
+		if (!copymsg){
+			cl_log(LOG_ERR, "msg copy failed");
+			return FALSE;
+		}
+		
+		t->client_status_msg_queue = 
+			g_list_append(t->client_status_msg_queue,
+				      copymsg);
+		if (ANYDEBUG){
+			cl_log(LOG_DEBUG,"one entry added to "
+			       "client_status_msg_queue"
+			       "for node %s", thisnode->nodename);
+		}
+		return FALSE;
+	}
+	
+
+	
+	return TRUE;
+}
+
+
 
 /*
  *	One client pointer per input FIFO.  It's indexed by file descriptor, so
@@ -177,7 +365,6 @@ extern GHashTable*	APIAuthorization;
  * oblivion by Alan Robertson <alanr@unix.sh>
  *
  */
-
 /*
  *	Monitor messages.  Pass them along to interested clients (if any)
  */
@@ -187,6 +374,7 @@ api_heartbeat_monitor(struct ha_msg *msg, int msgtype, const char *iface)
 	const char*	clientid;
 	client_proc_t*	client;
 	client_proc_t*	nextclient;
+	
 
 	/* This kicks out most messages, since debug clients are rare */
 
@@ -222,11 +410,31 @@ api_heartbeat_monitor(struct ha_msg *msg, int msgtype, const char *iface)
 			continue;
 		}
 
-		/* Is this one of the types of messages we're interested in? */
-
-		if ((msgtype & client->desired_types) != 0) {
-			api_send_client_msg(client, msg);
+		/* Is this one of the types of messages we're interested in?*/
+		
+		if ((msgtype & client->desired_types) != 0) {		       	
+			
+			if (should_msg_sendto_client(client, msg)){				
+				api_send_client_msg(client, msg);				
+			}else {
+				/*This happens when join/leave messages is
+				 *received but there are messages before 
+				 *that are missing. The join/leave messages
+				 *will be queued and not delivered until all
+				 *messages before them are received and 
+				 *delivered. 
+				 */
+				
+				/*do nothing*/
+			}
+			
 			if (client->removereason && !client->isindispatch) {
+				if (ANYDEBUG){
+					cl_log(LOG_DEBUG
+					,	"%s: client is %s"
+					,	__FUNCTION__
+					,	client->client_id);
+				}
 				api_remove_client_pid(client->pid
 				,	client->removereason);
 			}
@@ -313,6 +521,11 @@ api_signoff(const struct ha_msg* msg, struct ha_msg* resp
 			cl_log(LOG_DEBUG, "Signing client %ld off"
 			,	(long) client->pid);
 		}
+		if (client->seq_snapshot_table){
+			destroy_seq_snapshot_table(client->seq_snapshot_table);
+			client->seq_snapshot_table = NULL;
+		}
+		
 		client->removereason = API_SIGNOFF;
 		return I_API_IGN;
 }
@@ -386,6 +599,7 @@ api_nodestatus(const struct ha_msg* msg, struct ha_msg* resp
 {
 		const char *		cnode;
 		struct node_info *	node;
+		const char *		savedstat;
 
 		if ((cnode = ha_msg_value(msg, F_NODENAME)) == NULL
 		|| (node = lookup_node(cnode)) == NULL) {
@@ -397,11 +611,66 @@ api_nodestatus(const struct ha_msg* msg, struct ha_msg* resp
 			,	"api_nodestatus: cannot add field");
 			return I_API_IGN;
 		}
+		/* Give them the "real" (non-delayed) status */
+		if (node->saved_status_msg
+		&&	(savedstat
+		=	ha_msg_value(node->saved_status_msg, F_STATUS))) {
+			ha_msg_mod(resp, F_STATUS, savedstat);
+		}
 		return I_API_RET;
 }
 
 /**********************************************************************
- * API_NODESTATUS: Return the status of the given node
+ * API_NODEWEIGHT: Return the weight of the given node
+ *********************************************************************/
+
+static int
+api_nodeweight(const struct ha_msg* msg, struct ha_msg* resp
+,	client_proc_t* client, const char** failreason)
+{
+	const char *		cnode;
+	struct node_info *	node;
+
+	if ((cnode = ha_msg_value(msg, F_NODENAME)) == NULL
+	|| (node = lookup_node(cnode)) == NULL) {
+		*failreason = "EINVAL";
+		return I_API_BADREQ;
+	}
+	
+	if (ha_msg_add_int(resp, F_WEIGHT, node->weight) != HA_OK) {
+		cl_log(LOG_ERR
+		,	"api_nodeweight: cannot add field");
+		return I_API_IGN;
+	}
+	return I_API_RET;
+}
+
+/**********************************************************************
+ * API_NODESITE: Return the site of the given node
+ *********************************************************************/
+
+static int
+api_nodesite(const struct ha_msg* msg, struct ha_msg* resp
+,	client_proc_t* client, const char** failreason)
+{
+	const char *		cnode;
+	struct node_info *	node;
+
+	if ((cnode = ha_msg_value(msg, F_NODENAME)) == NULL
+	|| (node = lookup_node(cnode)) == NULL) {
+		*failreason = "EINVAL";
+		return I_API_BADREQ;
+	}
+	if (ha_msg_add(resp, F_SITE, node->site) != HA_OK) {
+		cl_log(LOG_ERR
+		,	"api_nodesite: cannot add field");
+		return I_API_IGN;
+	}
+	return I_API_RET;
+}
+
+/**********************************************************************
+ * API_NODETYPE: Return the type of the given node
  *********************************************************************/
 
 static int
@@ -562,12 +831,14 @@ api_clientstatus(const struct ha_msg* msg, struct ha_msg* resp
 {
 	const char *		cnode;
 	const char *		cname;
+	const char *		our_clientid;
 	struct node_info *	node;
 	struct ha_msg *		m;
 	int			ret = HA_FAIL;
 
 	if ((cnode = ha_msg_value(msg, F_NODENAME)) == NULL
 	|| (cname = ha_msg_value(msg, F_CLIENTNAME)) == NULL
+	|| (our_clientid = ha_msg_value(msg, F_FROMID)) == NULL
 	|| (node = lookup_node(cnode)) == NULL) {
 		*failreason = "EINVAL";
 		return I_API_BADREQ;
@@ -604,7 +875,8 @@ api_clientstatus(const struct ha_msg* msg, struct ha_msg* resp
 	if ((m = ha_msg_new(0)) == NULL
 	||	ha_msg_add(m, F_TYPE, T_QCSTATUS) != HA_OK
 	||	ha_msg_add(m, F_TO, cnode) != HA_OK
-	||	ha_msg_add(m, F_CLIENTNAME, cname) != HA_OK) {
+	||	ha_msg_add(m, F_CLIENTNAME, cname) != HA_OK
+	|| 	ha_msg_add(m, F_FROMID, our_clientid) != HA_OK) {
 
 		ha_log(LOG_ERR, "api_clientstatus: cannot add field");
 		*failreason = "ENOMEM";
@@ -621,6 +893,36 @@ api_clientstatus(const struct ha_msg* msg, struct ha_msg* resp
 	 */
 	return I_API_IGN;
 }
+/**********************************************************************
+ * API_NUM_NODES: Return the number of normal nodes
+ *********************************************************************/
+
+static int
+api_num_nodes(const struct ha_msg* msg, struct ha_msg* resp
+	      ,	client_proc_t* client, const char** failreason)
+{
+	int ret;
+	int num_nodes = 0;
+	int i;
+
+	for( i = 0; i < config->nodecount; i++){
+		if (config->nodes[i].nodetype == NORMALNODE_I){
+			num_nodes++;
+		}
+	}
+
+	ret = ha_msg_add_int(resp, F_NUMNODES, num_nodes);
+	if (ret != HA_OK){
+		cl_log(LOG_ERR, "%s: adding num_nodes field failed",
+		       __FUNCTION__);
+		*failreason= "adding msg field failed";
+		return I_API_BADREQ;
+	}
+	
+	return I_API_RET;
+
+}
+
 
 /**********************************************************************
  * API_GET_PARAMETER: Return the value of the given parameter...
@@ -653,7 +955,8 @@ api_get_resources (const struct ha_msg* msg, struct ha_msg* resp
 	const char *		ret;
 
 	if (!DoManageResources) {
-		*failreason = "EINVAL";
+		*failreason = "Resource is managed by crm."
+			"Use crm tool to query resource";
 		return I_API_BADREQ;
 	}
 
@@ -666,6 +969,112 @@ api_get_resources (const struct ha_msg* msg, struct ha_msg* resp
 	return I_API_RET;
 }
 
+
+static int
+api_get_uuid (const struct ha_msg* msg, struct ha_msg* resp,
+	      client_proc_t* client, const char** failreason)
+{
+	const char*	query_nodename;
+	cl_uuid_t	uuid;
+	
+	if ((query_nodename = ha_msg_value(msg, F_QUERYNAME))== NULL){
+		*failreason = "no query node name found";
+		return I_API_BADREQ;
+	}
+	
+	if (nodename2uuid(query_nodename, &uuid) != HA_OK){
+		return I_API_RET;
+	}
+	
+	if (cl_msg_moduuid(resp, F_QUERYUUID, 
+			   &uuid) != HA_OK){
+		cl_log(LOG_ERR, "api_get_uuid: cannnot add"
+		       F_QUERYUUID " field to message");
+		return I_API_RET;
+	}
+	
+	return I_API_RET;
+}
+
+
+static int
+api_get_nodename(const struct ha_msg* msg, struct ha_msg* resp,
+		 client_proc_t* client, const char** failreason)
+{
+	const char*	nodename;
+	cl_uuid_t	query_uuid;
+
+	if (cl_get_uuid(msg, F_QUERYUUID, &query_uuid) != HA_OK){
+		*failreason = "no query node name found";
+		return I_API_BADREQ;
+	}
+	
+	if ((nodename = uuid2nodename(&query_uuid)) == NULL){
+		cl_log(LOG_ERR, "api_get_nodename: nodename not found"
+		       " in map table");
+		return I_API_RET;
+	}
+	
+	if (ha_msg_mod(resp, F_QUERYNAME, 
+		       nodename) != HA_OK){
+		cl_log(LOG_ERR, "api_get_nodename: cannnot add"
+		       F_QUERYNAME " field to message");
+		return I_API_RET;
+	}
+	
+	return I_API_RET;	
+	
+}
+
+
+static int
+api_set_sendqlen(const struct ha_msg* msg, struct ha_msg* resp,
+		 client_proc_t* client, const char** failreason)
+{
+	int length;
+	int ret = ha_msg_value_int(msg, F_SENDQLEN, &length);
+	
+
+	if (ret != HA_OK){
+		cl_log(LOG_ERR, "api_set_sendqlen: getting field F_SENDQLEN failed");
+		return I_API_IGN;	
+	}
+
+	if (length <= 0){
+		cl_log(LOG_ERR, "api_set_sendqlen: invalid length value(%d)", 
+		       length);
+		return I_API_IGN;
+	}
+
+	cl_log(LOG_INFO, "the send queue length from heartbeat to client %s "
+	       "is set to %d",  client->client_id, length);
+	
+	client->chan->ops->set_send_qlen(client->chan,length); 
+
+
+	return I_API_IGN;
+	
+}
+
+static int
+add_client_gen(client_proc_t* client, struct ha_msg* msg)
+{
+	char buf[MAX_CLIENT_GEN];
+	
+	memset(buf, 0, MAX_CLIENT_GEN);
+	snprintf(buf, MAX_CLIENT_GEN, "%d", client->cligen);	
+	if (ha_msg_add(msg, F_CLIENT_GENERATION, buf) != HA_OK){
+		cl_log(LOG_ERR, "api_send_client_status: cannot add fields");
+		ha_msg_del(msg); msg=NULL;
+		return HA_FAIL;
+	}
+	
+	return HA_OK;
+	
+}
+
+
+
 /*
  * Process an API request message from one of our clients
  */
@@ -729,10 +1138,15 @@ api_process_request(client_proc_t* fromclient, struct ha_msg * msg)
  		}
 		if (DEBUGDETAILS) {
 			cl_log(LOG_DEBUG, "Sending API message to cluster...");
-			cl_log_message(msg);
+			cl_log_message(LOG_DEBUG, msg);
 		}
 
 		/* Mikey likes it! */
+		if (add_client_gen(fromclient, msg) != HA_OK){
+			cl_log(LOG_ERR, "api_process_request: "
+			       " add client generation to ha_msg failed ");			
+		}
+
 		if (send_cluster_msg(msg) != HA_OK) {
 			cl_log(LOG_ERR, "api_process_request: "
 			"cannot forward message to cluster");
@@ -826,7 +1240,7 @@ api_process_request(client_proc_t* fromclient, struct ha_msg * msg)
 bad_req:
 	cl_log(LOG_ERR, "api_process_request: bad request [%s]"
 	,	reqtype);
-	cl_log_message(msg);
+	cl_log_message(LOG_ERR, msg);
 	if (ha_msg_add(resp, F_APIRESULT, API_BADREQ) != HA_OK) {
 		cl_log(LOG_ERR
 		,	"api_process_request: cannot add field/11");
@@ -874,6 +1288,9 @@ process_registerevent(IPC_Channel* chan,  gpointer user_data)
 	,	chan, FALSE
 	,	APIclients_input_dispatch
 	,	client, G_remove_client);
+	G_main_setdescription((GSource*)client->gsource, "API client");
+	G_main_setmaxdispatchdelay((GSource*)client->gsource, config->heartbeat_ms);
+	G_main_setmaxdispatchtime((GSource*)client->gsource, 100);
 	if (ANYDEBUG) {
 		cl_log(LOG_DEBUG
 		,	"client->gsource = 0x%lx"
@@ -889,6 +1306,103 @@ process_registerevent(IPC_Channel* chan,  gpointer user_data)
 	}
 }
 
+static void     
+destroy_pair(gpointer key, gpointer value, gpointer user_data)
+{
+	if(value){
+		cl_free(value);
+	}
+}
+static void
+destroy_seq_snapshot_table(GHashTable* table) 
+{
+	if (ANYDEBUG){
+		cl_log(LOG_DEBUG,
+		       "Destroying seq snapshot hash table");
+	}
+
+	if(table){
+		g_hash_table_foreach(table, destroy_pair, NULL);
+		g_hash_table_destroy(table);
+	}
+	return ;
+}
+static int
+create_seq_snapshot_table(GHashTable** ptable) 
+{
+	GHashTable*		table = NULL;
+	int			i;
+	
+	if ( !ptable){
+		cl_log(LOG_ERR, "create_seq_snapshot_table: "
+		       "nvalid arguments");
+		return HA_FAIL;		
+	}
+	
+	*ptable = NULL;
+	for (i = 0 ; i < config->nodecount; i++){
+	
+		struct node_info*	node = &config->nodes[i];
+		struct seqtrack*	t = &node->track;
+
+
+		if (cl_uuid_is_null(&node->uuid)){
+			continue;
+		}
+
+		if (t->nmissing > 0){
+			struct seq_snapshot* snapshot;
+			
+			snapshot = (struct seq_snapshot*) 
+				cl_malloc(sizeof(struct seq_snapshot));
+			
+			if (snapshot == NULL){
+				cl_log(LOG_ERR, "allocating memory for"
+				       " seq_snapshot failed");
+				return HA_FAIL;
+			}
+		
+			snapshot->last_seq = t->last_seq;
+			snapshot->generation = t->generation;
+
+			if (table == NULL){
+				if (ANYDEBUG){
+					cl_log(LOG_DEBUG,
+					       "Creating seq snapshot hash table");
+				}
+				table = g_hash_table_new(uuid_hash, uuid_equal);
+				if (table == NULL){
+					cl_log(LOG_ERR, "creating hashtable for"
+					       " seq_snapshot failed");
+					return HA_FAIL;
+				}
+			}
+			if (ANYDEBUG){
+				cl_log(LOG_DEBUG,
+				       "Creating one entry in seq snapshot hash table"
+				       "for node %s", node->nodename);
+			}
+			
+			g_hash_table_insert(table, &node->uuid, snapshot);
+			
+		}else{
+			if (ANYDEBUG){
+				cl_log(LOG_DEBUG,
+				       "create_seq_snapshot_table:"
+				       "no missing packets found for "
+				       "node %s", node->nodename);
+			}	
+		}
+	}
+	
+	*ptable = table;
+	
+	return HA_OK;
+}	
+
+
+
+
 /*
  *	Register a new client.
  */
@@ -906,13 +1420,17 @@ api_process_registration_msg(client_proc_t* client, struct ha_msg * msg)
 	char		keepalive[64];
 	char		logfacility[64];
 
+
+	/*set the client generation*/
+	client->cligen =  client_generation++;
+
 	if (msg == NULL
 	||	(msgtype = ha_msg_value(msg, F_TYPE)) == NULL
 	||	(reqtype = ha_msg_value(msg, F_APIREQ)) == NULL
 	||	strcmp(msgtype, T_APIREQ) != 0
 	||	strcmp(reqtype, API_SIGNON) != 0)  {
 		cl_log(LOG_ERR, "api_process_registration_msg: bad message");
-		cl_log_message(msg);
+		cl_log_message(LOG_ERR, msg);
 		goto del_msg;
 	}
 	fromid = ha_msg_value(msg, F_FROMID);
@@ -954,7 +1472,8 @@ api_process_registration_msg(client_proc_t* client, struct ha_msg * msg)
 	 */
 	if (!api_add_client(client, msg)) {
 		cl_log(LOG_ERR
-		,	"api_process_registration_msg: cannot add client(1)");
+		       ,	"api_process_registration_msg: cannot add client(%s)"
+		       ,	client->client_id);
 	}
 
 	/* Make sure we can find them in the table... */
@@ -970,6 +1489,18 @@ api_process_registration_msg(client_proc_t* client, struct ha_msg * msg)
 		goto del_rsp_and_msg;
 		return;
 	}
+
+	/*everything goes well, 
+	  now create a table to record sequence/generation number
+	  for each node if necessary*/
+	
+	client->seq_snapshot_table = NULL;
+	if (create_seq_snapshot_table(&client->seq_snapshot_table) != HA_OK){
+		cl_log(LOG_ERR, "api_process_registration_msg: "
+		       " creating seq snapshot table failed");
+		return;
+	}
+
 	if (ha_msg_mod(resp, F_APIRESULT, API_OK) != HA_OK) {
 		cl_log(LOG_ERR
 		,	"api_process_registration_msg: cannot add field/4");
@@ -995,12 +1526,6 @@ api_process_registration_msg(client_proc_t* client, struct ha_msg * msg)
 		,	(client->iscasual? "'casual'" : client->client_id));
 	}
 	api_send_client_msg(client, resp);
-	/* What to do with 'resp'.  Do we need it any more?? FIXME!! */
-	/* I think we don't need it anymore when go here, or memory leak
-	   will be very severe. You don't test it;-)	
-	goto del_msg;
-	*/
-
 del_rsp_and_msg:
 	if (resp != NULL) {
 		ha_msg_del(resp); resp=NULL;
@@ -1011,12 +1536,13 @@ del_msg:
 	}
 }
 
+
+
 static void
 api_send_client_status(client_proc_t* client, const char * status
 ,	const char *	reason)
 {
-	struct ha_msg*	msg;
-
+	struct ha_msg*		msg;
 	if (client->iscasual) {
 		return;
 	}
@@ -1028,7 +1554,7 @@ api_send_client_status(client_proc_t* client, const char * status
 		cl_log(LOG_ERR, "api_send_client_status: out of memory/1");
 		return;
 	}
-
+	
 	if (ha_msg_add(msg, F_TYPE, T_APICLISTAT) != HA_OK
 	||	ha_msg_add(msg, F_STATUS, status) != HA_OK
 	||	ha_msg_add(msg, F_FROMID, client->client_id) != HA_OK
@@ -1040,13 +1566,21 @@ api_send_client_status(client_proc_t* client, const char * status
 		ha_msg_del(msg); msg=NULL;
 		return;
 	}
+	
+	if (add_client_gen(client, msg) != HA_OK){
+		cl_log(LOG_ERR, "api_send_client_status: cannot add fields");
+		ha_msg_del(msg); msg=NULL;
+		return;
+	}
+	
 	if (strcmp(status, LEAVESTATUS) == 0) {
 		/* Make sure they know they're signed off... */
 		api_send_client_msg(client, msg);
 	}
+	
 	if (send_cluster_msg(msg) != HA_OK) {
 		cl_log(LOG_ERR, "api_send_client_status: "
-			"cannot send message to cluster");
+		       "cannot send message to cluster");
 	}
 	msg = NULL;
 }
@@ -1059,7 +1593,11 @@ api_send_client_msg(client_proc_t* client, struct ha_msg *msg)
 {
 	if (msg2ipcchan(msg, client->chan) != HA_OK) {
 		if (!client->removereason) {
-			client->removereason = "sendfail";
+			if (client->chan->failreason[0] == EOS){
+				client->removereason = "sendfail";
+			}else {
+				client->removereason = client->chan->failreason;
+			}
 		}
 	}
 
@@ -1082,13 +1620,18 @@ api_remove_client_pid(pid_t c_pid, const char * reason)
 	client_proc_t* 	client;
 
 	snprintf(cpid, sizeof(cpid)-1, "%d", c_pid);
-	if ((client = find_client(cpid, NULL)) == NULL) {
+	if ((client = find_client(NULL, cpid)) == NULL) {
 		return 0;
 	}
 
 	client->removereason = reason;
+	while (client->chan->recv_queue->current_qlen > 0
+	&&	(!reason || strcmp(reason, API_SIGNOFF) != 0)) {
+		ProcessAnAPIRequest(client);
+	}
+
 	G_main_del_IPC_Channel(client->gsource);
-	client->gsource = NULL;
+	/* Should trigger G_remove_client (below) */
 	return 1;
 }
 static void
@@ -1123,6 +1666,8 @@ api_remove_client_int(client_proc_t* req, const char * reason)
 	client_proc_t*	prev = NULL;
 	client_proc_t*	client;
 
+		
+
 	--total_client_count;
 
 	if ((req->desired_types & DEBUGTREATMENTS) != 0) {
@@ -1146,31 +1691,25 @@ api_remove_client_int(client_proc_t* req, const char * reason)
 				prev->next = client->next;
 			}
 
-#if MAKEITCRASH
-			/* Drop the source - that will destroy the 'chan' */
-			if (client->gsource) {
-				G_main_del_IPC_Channel(client->gsource);
-			}
-#endif
 			break;
 		}
 		prev = client;
 	}
+	
 
-       if (req == client){
-               
-               api_send_client_status(req, LEAVESTATUS, reason);
-               
-               /* Zap! */
-               memset(client, 0, sizeof(*client));
-               ha_free(client); client = NULL;
-       }else{
-               cl_log(LOG_ERR, "api_remove_client_int: could not find pid [%ld]"
-                      ,        (long) req->pid);
-       }
-       
-       return;
+	if (req == client){
+		
+		api_send_client_status(req, LEAVESTATUS, reason);
+		/* Zap! */
+		memset(client, 0, sizeof(*client));
+		cl_free(client); client = NULL;
 
+	}else{
+		cl_log(LOG_ERR,	"api_remove_client_int: could not find pid [%ld]"
+		       ,	(long) req->pid);
+	}
+	
+	return;
 }
 
 
@@ -1184,6 +1723,7 @@ api_check_client_credentials(client_proc_t* client, uid_t uid, gid_t gid)
 	int		one = 1;
 	gboolean	result = TRUE;
 
+	int auth_result = IPC_FAIL;
 	GHashTable*	uidlist
 	=		g_hash_table_new(g_direct_hash, g_direct_equal);
 
@@ -1191,9 +1731,12 @@ api_check_client_credentials(client_proc_t* client, uid_t uid, gid_t gid)
 	g_hash_table_insert(uidlist, GUINT_TO_POINTER(id), &one);
 	auth.uid = uidlist;
 	auth.gid = NULL;
-	if (client->chan->ops->verify_auth(client->chan, &auth) != IPC_OK) {
+	auth_result = client->chan->ops->verify_auth(client->chan, &auth);
+
+	if (auth_result != IPC_OK) {
 		result = FALSE;
-	}else {
+
+	} else {
 		GHashTable*	gidlist
 		=		g_hash_table_new(g_direct_hash
 		,		g_direct_equal);
@@ -1201,13 +1744,17 @@ api_check_client_credentials(client_proc_t* client, uid_t uid, gid_t gid)
 		g_hash_table_insert(gidlist, GUINT_TO_POINTER(id), &one);
 		auth.uid = NULL;
 		auth.gid = gidlist;
-		if (client->chan->ops->verify_auth(client->chan, &auth)
-		!=	IPC_OK) {
-			result =  FALSE;
+
+		auth_result = client->chan->ops->verify_auth(
+			client->chan, &auth);
+		
+		if (auth_result != IPC_OK && auth_result != IPC_BROKEN) {
+			result = FALSE;
 		}
 		g_hash_table_destroy(gidlist);
 	}
 	g_hash_table_destroy(uidlist);
+
 	return result;
 }
 
@@ -1230,8 +1777,8 @@ api_add_client(client_proc_t* client, struct ha_msg* msg)
 	const char *	cuid = NULL;
 	long		luid = -1;
 	long		lgid = -1;
-	uid_t		uid = (uid_t)-1;
-	gid_t		gid = (gid_t)-1;
+	int		uid = -1;
+	int		gid = -1;
 
 	
 	if ((cpid = ha_msg_value(msg, F_PID)) != NULL) {
@@ -1268,12 +1815,14 @@ api_add_client(client_proc_t* client, struct ha_msg* msg)
 		client->iscasual = 1;
 	}
 
+	/* Encourage better realtime behavior by heartbeat */
+	client->chan->ops->set_recv_qlen(client->chan, 0);
 
 	if ((cuid = ha_msg_value(msg, F_UID)) == NULL
 	||	(cgid = ha_msg_value(msg, F_GID)) == NULL
 	||	sscanf(cuid, "%ld", &luid) != 1
 	||	sscanf(cgid, "%ld", &lgid) != 1) {
-		cl_log_message(msg);
+		cl_log_message(LOG_ERR, msg);
 		client->removereason = "invalid id info";
 		cl_log(LOG_ERR, "Client user/group id is incorrect"
 		" [%s] => %ld [%s] => %ld"
@@ -1297,7 +1846,6 @@ api_add_client(client_proc_t* client, struct ha_msg* msg)
 		cl_log(LOG_WARNING
 		,	"Client [%s] pid %d failed authorization [%s]"
 		,	client->client_id, pid, client->removereason);
-		api_send_client_status(client, LEAVESTATUS, "auth failure");
 		return FALSE;
 	}
 	return TRUE;
@@ -1308,7 +1856,8 @@ api_check_client_authorization(client_proc_t* client)
 {
 	gpointer	gauth = NULL;
 	IPC_Auth*	auth;
-
+	int auth_result = IPC_FAIL;
+	
 	/* If client is casual, or type of client not in authorization table
 	 * then default to the "default" authorization category.
 	 * otherwise, use the client type's authorization list
@@ -1327,8 +1876,8 @@ api_check_client_authorization(client_proc_t* client)
 			client->removereason = "no default client auth";
 			return FALSE;
 		}
-		
 	}
+	
 	auth = gauth;
 	if ((long)auth->gid == (long)-1L) {
 		cl_log(LOG_DEBUG, "Darn!  -1 gid ptr in api_check_client_authorization");
@@ -1341,8 +1890,12 @@ api_check_client_authorization(client_proc_t* client)
 		,	client->client_id
 		,	(long)client->uid, (long)client->gid);
 	}
-	if (auth == NULL
-	||	client->chan->ops->verify_auth(client->chan, auth) == IPC_OK) {
+
+	auth_result = client->chan->ops->verify_auth(
+			client->chan, auth);
+	
+	if (auth_result == IPC_OK) {
+#ifndef GETPID_INCONSISTENT
 		if (client->chan->farside_pid > 0) {
 			if (client->chan->farside_pid != client->pid) {
 				client->removereason = "pid mismatch";
@@ -1353,8 +1906,13 @@ api_check_client_authorization(client_proc_t* client)
 				return FALSE;
 			}
 		}
+#endif
+		return TRUE;
+
+	} else if(auth_result == IPC_BROKEN) {
 		return TRUE;
 	}
+	
 	client->removereason = "client failed authorization";
 	return FALSE;
 }
@@ -1392,6 +1950,7 @@ APIclients_input_dispatch(IPC_Channel* chan, gpointer user_data)
 	client_proc_t*	client = user_data;
 	gboolean	ret = TRUE;
 
+
 	if (DEBUGDETAILS) {
 		cl_log(LOG_DEBUG
 		,	"APIclients_input_dispatch() {");
@@ -1410,9 +1969,7 @@ APIclients_input_dispatch(IPC_Channel* chan, gpointer user_data)
 
 	/* Process a single API client request */
 	client->isindispatch = TRUE;
-	while (ProcessAnAPIRequest(client)) {
-		/* Do nothing */;
-	}
+	ProcessAnAPIRequest(client);
 	client->isindispatch = FALSE;
 
 	if (client->removereason) {
@@ -1431,6 +1988,46 @@ getout:
 }
 
 
+static gboolean all_clients_running = TRUE;
+gboolean
+all_clients_pause(void)
+{
+	client_proc_t* client;
+
+	if (!all_clients_running ){
+		return TRUE;
+	}
+	
+	cl_log(LOG_INFO, "all clients are now paused");
+		
+	for (client=client_list; client != NULL; client=client->next) {
+		G_main_IPC_Channel_pause(client->gsource);
+		
+	}	
+	all_clients_running = FALSE;
+	return TRUE;
+}
+
+gboolean
+all_clients_resume(void)
+{
+	client_proc_t* client;
+	
+	if (all_clients_running ){
+		return TRUE;
+	}
+	
+	cl_log(LOG_INFO, "all clients are now resumed");
+	
+	for (client=client_list; client != NULL; client=client->next) {
+		G_main_IPC_Channel_resume(client->gsource);		
+	}	
+	
+	all_clients_running = TRUE;
+	
+	return TRUE;
+}
+
 gboolean
 ProcessAnAPIRequest(client_proc_t*	client)
 {
@@ -1447,19 +2044,20 @@ ProcessAnAPIRequest(client_proc_t*	client)
 	if (!client->chan->ops->is_message_pending(client->chan)) {
 		goto getout;
 	}
-
-
+	
 	/* See if we can read the message */
-	if ((msg = msgfromIPC_noauth(client->chan)) == NULL) {
-
+	if ((msg = msgfromIPC(client->chan, 0)) == NULL) {
+		
 		/* EOF? */
-		if (client->chan->ch_status == IPC_DISCONNECT) {
+		if (!IPC_ISRCONN(client->chan)) {
 			if (ANYDEBUG) {
 				cl_log(LOG_DEBUG
 				,	"EOF from client pid %ld"
 				,	(long)client->pid);
 			}
-			client->removereason = "EOF";
+			if (!client->removereason) {
+				client->removereason = "EOF";
+			}
 			goto getout;
 		}
 
@@ -1495,6 +2093,14 @@ ProcessAnAPIRequest(client_proc_t*	client)
 	}
 	msg = NULL;
 	rc = TRUE;
+	
+	if (!all_clients_running){
+		/* This is a new client,
+		 * this allows a client to sign on but further action will be blocked
+		 */
+		G_main_IPC_Channel_pause(client->gsource);
+		rc = FALSE;
+	}
 
 getout:
 	/* May have gotten a message from 'client' */
