@@ -1,4 +1,3 @@
-/* $Id: hb_signal.c,v 1.9.2.1 2004/04/20 05:05:28 alan Exp $ */
 /*
  * hb_signal.c: signal handling routines to be used by Heartbeat
  *
@@ -7,24 +6,23 @@
  * Derived from code in heartbeat.c in this tree
  * Copyright (C) 2000 Alan Robertson <alanr@unix.sh>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- * 
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #define _USE_BSD
-#include <portability.h>
+#include <lha_internal.h>
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/resource.h>
@@ -46,7 +44,7 @@
 #include <hb_proc.h>
 #include <heartbeat_private.h>
 #include <heartbeat.h>
-#include <setproctitle.h>
+#include <clplumbing/GSource.h>
 #include <pils/plugin.h>
 #include <test.h>
 
@@ -192,25 +190,25 @@ __hb_signal_debug_action(int sig)
 
 	switch(sig) {
 		case SIGUSR1:
-			++debug;
+			++debug_level;
 			break;
 
 		case SIGUSR2:
-			if (debug > 0) {
-				--debug;
+			if (debug_level > 0) {
+				--debug_level;
 			}else{
-				debug=0;
+				debug_level=0;
 			}
 			break;
 	}
 
- 	PILSetDebugLevel(PluginLoadingSystem, NULL, NULL , debug);
+ 	PILSetDebugLevel(PluginLoadingSystem, NULL, NULL , debug_level);
 	{
 		static char cdebug[8];
-		snprintf(cdebug, sizeof(debug), "%d", debug);
+		snprintf(cdebug, sizeof(debug_level), "%d", debug_level);
 		setenv(HADEBUGVAL, cdebug, TRUE);
 	}
-	if (debug <= 0) {
+	if (debug_level <= 0) {
 		unsetenv(HADEBUGVAL);
 	}
 }
@@ -247,14 +245,14 @@ hb_signal_debug_usr2_action(void)
 static void
 __parent_hb_signal_debug_action(int sig)
 {
-	int	olddebug = debug;
+	int	olddebug = debug_level;
 
 	__hb_signal_debug_action(sig);
 	hb_signal_signal_all(sig);
 
-	ha_log(LOG_DEBUG, "debug now set to %d [pid %d]", debug
+	ha_log(LOG_DEBUG, "debug now set to %d [pid %d]", debug_level
 	,	(int) getpid());
-	if (debug == 1 && olddebug == 0) {
+	if (debug_level == 1 && olddebug == 0) {
 		hb_versioninfo();
 		hb_dump_proc_stats(curproc);
 	}
@@ -479,7 +477,6 @@ hb_signal_set_common(sigset_t *set)
 	,	{SIGALRM,	hb_signal_false_alarm_handler,	1}
 	,	{SIGUSR1,	hb_signal_debug_usr1_handler,	1}
 	,	{SIGUSR2,	hb_signal_debug_usr2_handler,	1}
-	,	{SIGCHLD,	hb_signal_reaper_handler,	1}
 	,	{0,		0,				0}
 	};
 
@@ -583,12 +580,44 @@ hb_signal_set_read_child(sigset_t *set)
 int
 hb_signal_set_fifo_child(sigset_t *set)
 {
-	if (hb_signal_set_common(set) < 0) {
-		ha_log(LOG_ERR, "hb_signal_set_read_child(): "
+	sigset_t *use_set;
+	sigset_t our_set;
+
+	const cl_signal_mode_t mode [] = { 
+		{SIGALRM,	hb_ignoresig,	1}
+	,	{0,		0,		0}
+	};
+
+	if (set) {
+		use_set = set;
+	}else{
+		use_set = &our_set;
+
+		if (CL_SIGEMPTYSET(use_set) < 0) {
+			ha_log(LOG_ERR, "hb_signal_set_write_child(): "
+				"CL_SIGEMPTYSET(): %s", strerror(errno));
+			return(-1);
+		}
+	}
+
+	if (hb_signal_set_common(use_set) < 0) {
+		ha_log(LOG_ERR, "hb_signal_set_fifo_child(): "
 			"hb_signal_set_common()");
 		return(-1);
 	}
 
+	if (cl_signal_set_handler_mode(mode, use_set) < 0) {
+		ha_log(LOG_ERR, "hb_signal_set_fifo_child(): "
+			"cl_signal_set_handler_mode()");
+		return(-1);
+	}
+
+	if (cl_signal_set_handler_mode(mode, use_set) < 0) {
+		ha_log(LOG_ERR, "%s(): cl_signal_set_handler_mode() failed."
+		,	__FUNCTION__);
+		return(-1);
+	}
+
 	hb_signal_process_pending_set_mask_set(set);
 
 	return(0);
@@ -632,7 +661,8 @@ hb_signal_set_master_control_process(sigset_t *set)
 			"cl_signal_set_handler_mode()");
 		return(-1);
 	}
-
+	
+	set_sigchld_proctrack(G_PRIORITY_HIGH);
 	hb_signal_process_pending_set_mask_set(use_set);
 
 	return(0);
