@@ -1,4 +1,3 @@
-/* $Id: send_arp.c,v 1.7.2.3 2004/09/14 15:42:32 alan Exp $ */
 /* 
  * send_arp
  * 
@@ -13,33 +12,29 @@
 
  * Copyright (C) 2001 Matt Soffen <matt@soffen.com>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- * 
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <portability.h>
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include <lha_internal.h>
 
 #include <limits.h>
 #include <libnet.h>
-#include <syslog.h>
 #include <libgen.h>
 #include <clplumbing/timers.h>
 #include <clplumbing/cl_signal.h>
+#include <clplumbing/cl_log.h>
 
 #ifdef HAVE_LIBNET_1_0_API
 #	define	LTYPE	struct libnet_link_int
@@ -48,19 +43,46 @@
 #	define	LTYPE	libnet_t
 #endif
 
-#define PIDDIR       HA_VARLIBDIR "/" PACKAGE "/rsctmp/send_arp"
+#define PIDDIR       HA_VARRUNDIR "/" PACKAGE "/rsctmp/send_arp"
 #define PIDFILE_BASE PIDDIR "/send_arp-"
 
 static int send_arp(LTYPE* l, u_long ip, u_char *device, u_char mac[6]
 ,	u_char *broadcast, u_char *netmask, u_short arptype);
 
 static char print_usage[]={
-	"send_arp: sends out custom ARP packet. packetfactory.net\n"
-	"\tusage: send_arp [-i repeatinterval-ms] [-r repeatcount]"
-	" [-p pidfile] device src_ip_addr src_hw_addr broadcast_ip_addr netmask\n"
-	"\tIf src_hw_addr is \"auto\" then the address of device will be used"
+"send_arp: sends out custom ARP packet.\n"
+"  usage: send_arp [-i repeatinterval-ms] [-r repeatcount] [-p pidfile] \\\n"
+"              device src_ip_addr src_hw_addr broadcast_ip_addr netmask\n"
+"\n"
+"  where:\n"
+"    repeatinterval-ms: timing, in milliseconds of sending arp packets\n"
+"      For each ARP announcement requested, a pair of ARP packets is sent,\n"
+"      an ARP request, and an ARP reply. This is becuse some systems\n"
+"      ignore one or the other, and this combination gives the greatest\n"
+"      chance of success.\n"
+"\n"
+"      Each time an ARP is sent, if another ARP will be sent then\n"
+"      the code sleeps for half of repeatinterval-ms.\n"
+"\n"
+"    repeatcount: how many pairs of ARP packets to send.\n"
+"                 See above for why pairs are sent\n"
+"\n"
+"    pidfile: pid file to use\n"
+"\n"
+"    device: netowrk interace to use\n"
+"\n"
+"    src_ip_addr: source ip address\n"
+"\n"
+"    src_hw_addr: source hardware address.\n"
+"                 If \"auto\" then the address of device\n"
+"\n"
+"    broadcast_ip_addr: ignored\n"
+"\n"
+"    netmask: ignored\n"
 };
 
+static const char * SENDARPNAME = "send_arp";
+
 static void convert_macaddr (u_char *macaddr, u_char enet_src[6]);
 static int get_hw_addr(char *device, u_char mac[6]);
 int write_pid_file(const char *pidfilename);
@@ -73,6 +95,14 @@ int create_pid_directory(const char *piddirectory);
 #	define LIBNET_ERRBUF_SIZE 256
 #endif
 
+
+/* 
+ * For use logd, should keep identical with the same const variables defined
+ * in heartbeat.h.
+ */
+#define ENV_PREFIX "HA_"
+#define KEY_LOGDAEMON   "use_logd"
+
 static void
 byebye(int nsig)
 {
@@ -102,10 +132,14 @@ main(int argc, char *argv[])
 	char    pidfilenamebuf[64];
 	char    *pidfilename = NULL;
 
-	CL_SIGINTERRUPT(SIGTERM, 1);
 	CL_SIGNAL(SIGTERM, byebye);
+	CL_SIGINTERRUPT(SIGTERM, 1);
 
-	openlog("send_arp", LOG_CONS | LOG_PID, LOG_USER);
+        cl_log_set_entity(SENDARPNAME);
+        cl_log_enable_stderr(TRUE);
+        cl_log_set_facility(LOG_USER);
+        /* Use logd if it's enabled by heartbeat */
+        cl_inherit_use_logd(ENV_PREFIX ""KEY_LOGDAEMON, 0);
 
 	while ((flag = getopt(argc, argv, "i:r:p:")) != EOF) {
 		switch(flag) {
@@ -119,13 +153,13 @@ main(int argc, char *argv[])
 		case 'p':	pidfilename= optarg;
 				break;
 
-		default:	fprintf(stderr, "usage: %s\n\n", print_usage);
+		default:	fprintf(stderr, "%s\n\n", print_usage);
 				return 1;
 				break;
 		}
 	}
 	if (argc-optind != 5) {
-		fprintf(stderr, "usage: %s\n\n", print_usage);
+		fprintf(stderr, "%s\n\n", print_usage);
 		return 1;
 	}
 
@@ -147,7 +181,7 @@ main(int argc, char *argv[])
 		if (snprintf(pidfilenamebuf, sizeof(pidfilenamebuf), "%s%s", 
 					PIDFILE_BASE, ipaddr) >= 
 				(int)sizeof(pidfilenamebuf)) {
-			syslog(LOG_INFO, "Pid file truncated");
+			cl_log(LOG_INFO, "Pid file truncated");
 			return EXIT_FAILURE;
 		}
 		pidfilename = pidfilenamebuf;
@@ -158,27 +192,35 @@ main(int argc, char *argv[])
 	}
 
 #if defined(HAVE_LIBNET_1_0_API)
+#ifdef ON_DARWIN
+	if ((ip = libnet_name_resolve((unsigned char*)ipaddr, 1)) == -1UL) {
+#else
 	if ((ip = libnet_name_resolve(ipaddr, 1)) == -1UL) {
-		syslog(LOG_ERR, "Cannot resolve IP address [%s]", ipaddr);
+#endif
+		cl_log(LOG_ERR, "Cannot resolve IP address [%s]", ipaddr);
 		unlink(pidfilename);
 		return EXIT_FAILURE;
 	}
 
 	l = libnet_open_link_interface(device, errbuf);
 	if (!l) {
-		syslog(LOG_ERR, "libnet_open_link_interface on %s: %s"
+		cl_log(LOG_ERR, "libnet_open_link_interface on %s: %s"
 		,	device, errbuf);
 		unlink(pidfilename);
 		return EXIT_FAILURE;
 	}
 #elif defined(HAVE_LIBNET_1_1_API)
 	if ((l=libnet_init(LIBNET_LINK, device, errbuf)) == NULL) {
-		syslog(LOG_ERR, "libnet_init failure on %s", device);
+		cl_log(LOG_ERR, "libnet_init failure on %s: %s", device, errbuf);
 		unlink(pidfilename);
 		return EXIT_FAILURE;
 	}
+#ifdef ON_DARWIN
+ 	if ((signed)(ip = libnet_name2addr4(l, (unsigned char *)ipaddr, 1)) == -1) {
+#else
 	if ((signed)(ip = libnet_name2addr4(l, ipaddr, 1)) == -1) {
-		syslog(LOG_ERR, "Cannot resolve IP address [%s]", ipaddr);
+#endif
+		cl_log(LOG_ERR, "Cannot resolve IP address [%s]", ipaddr);
 		unlink(pidfilename);
 		return EXIT_FAILURE;
 	}
@@ -188,14 +230,14 @@ main(int argc, char *argv[])
 
 	if (!strcasecmp(macaddr, AUTO_MAC_ADDR)) {
 		if (get_hw_addr(device, src_mac) < 0) {
-			 syslog(LOG_ERR, "Cannot find mac address for %s", 
+			 cl_log(LOG_ERR, "Cannot find mac address for %s", 
 					 device);
 			 unlink(pidfilename);
 			 return EXIT_FAILURE;
 		}
 	}
 	else {
-		convert_macaddr(macaddr, src_mac);
+		convert_macaddr((unsigned char *)macaddr, src_mac);
 	}
 
 /*
@@ -204,14 +246,16 @@ main(int argc, char *argv[])
  * done by Masaki Hasegawa <masaki-h@pp.iij4u.or.jp> and his colleagues.
  */
 	for (j=0; j < repeatcount; ++j) {
-		c = send_arp(l, ip, device, src_mac, broadcast, 
-				netmask, ARPOP_REQUEST);
+		c = send_arp(l, ip, (unsigned char*)device, src_mac
+			, (unsigned char*)broadcast, (unsigned char*)netmask
+			, ARPOP_REQUEST);
 		if (c < 0) {
 			break;
 		}
 		mssleep(msinterval / 2);
-		c = send_arp(l, ip, device, src_mac, broadcast, 
-				netmask, ARPOP_REPLY);
+		c = send_arp(l, ip, (unsigned char*)device, src_mac
+			, (unsigned char *)broadcast
+			, (unsigned char *)netmask, ARPOP_REPLY);
 		if (c < 0) {
 			break;
 		}
@@ -241,7 +285,7 @@ convert_macaddr (u_char *macaddr, u_char enet_src[6])
 		bits[1] = macaddr[pos++];
 		bits[2] = '\0';
 
-		enet_src[i] = strtol(bits, (char **)NULL, 16);
+		enet_src[i] = strtol((const char *)bits, (char **)NULL, 16);
 	}
 
 }
@@ -254,20 +298,15 @@ get_hw_addr(char *device, u_char mac[6])
 	struct libnet_link_int  *network;
 	char                    err_buf[LIBNET_ERRBUF_SIZE];
 
-	/* Get around bad prototype for libnet_error() */
-	char errmess1 [] = "libnet_open_link_interface: %s\n";
-	char errmess2 [] = "libnet_get_hwaddr: %s\n";
-
-
 	network = libnet_open_link_interface(device, err_buf);
 	if (!network) {
-		libnet_error(LIBNET_ERR_FATAL, errmess1, err_buf);
+		fprintf(stderr, "libnet_open_link_interface: %s\n", err_buf);
 		return -1;
 	}
 
 	mac_address = libnet_get_hwaddr(network, device, err_buf);
 	if (!mac_address) {
-		libnet_error(LIBNET_ERR_FATAL, errmess2, err_buf);
+		fprintf(stderr, "libnet_get_hwaddr: %s\n", err_buf);
 		return -1;
 	}
 
@@ -354,22 +393,22 @@ send_arp(struct libnet_link_int *l, u_long ip, u_char *device, u_char *macaddr,
 
 
 	if (libnet_init_packet(LIBNET_ARP_H + LIBNET_ETH_H, &buf) == -1) {
-	syslog(LOG_ERR, "libnet_init_packet memory:");
+	cl_log(LOG_ERR, "libnet_init_packet memory:");
 		return -1;
 	}
 
 	/* Convert ASCII Mac Address to 6 Hex Digits. */
 
 	/* Ethernet header */
-	if (get_hw_addr(device, device_mac) < 0) {
-		syslog(LOG_ERR, "Cannot find mac address for %s",
+	if (get_hw_addr((char*)device, device_mac) < 0) {
+		cl_log(LOG_ERR, "Cannot find mac address for %s",
 				device);
 		return -1;
 	}
 
 	if (libnet_build_ethernet(bcast_mac, device_mac, ETHERTYPE_ARP, NULL, 0
 	,	buf) == -1) {
-		syslog(LOG_ERR, "libnet_build_ethernet failed:");
+		cl_log(LOG_ERR, "libnet_build_ethernet failed:");
 		libnet_destroy_packet(&buf);
 		return -1;
 	}
@@ -381,7 +420,7 @@ send_arp(struct libnet_link_int *l, u_long ip, u_char *device, u_char *macaddr,
 		target_mac = macaddr;
 	}
 	else {
-		syslog(LOG_ERR, "unkonwn arptype:");
+		cl_log(LOG_ERR, "unkonwn arptype:");
 		return -1;
 	}
 
@@ -400,14 +439,14 @@ send_arp(struct libnet_link_int *l, u_long ip, u_char *device, u_char *macaddr,
 		NULL,				/* Payload */
 		0,				/* Payload length */
 		buf + LIBNET_ETH_H) == -1) {
-	        syslog(LOG_ERR, "libnet_build_arp failed:");
+	        cl_log(LOG_ERR, "libnet_build_arp failed:");
 		libnet_destroy_packet(&buf);
 		return -1;
 	}
 
-	n = libnet_write_link_layer(l, device, buf, LIBNET_ARP_H + LIBNET_ETH_H);
+	n = libnet_write_link_layer(l, (char*)device, buf, LIBNET_ARP_H + LIBNET_ETH_H);
 	if (n == -1) {
-		syslog(LOG_ERR, "libnet_build_ethernet failed:");
+		cl_log(LOG_ERR, "libnet_build_ethernet failed:");
 	}
 
 	libnet_destroy_packet(&buf);
@@ -435,7 +474,7 @@ send_arp(libnet_t* lntag, u_long ip, u_char *device, u_char macaddr[6], u_char *
 		target_mac = macaddr;
 	}
 	else {
-		syslog(LOG_ERR, "unkonwn arptype:");
+		cl_log(LOG_ERR, "unkonwn arptype:");
 		return -1;
 	}
 
@@ -456,26 +495,26 @@ send_arp(libnet_t* lntag, u_long ip, u_char *device, u_char macaddr[6], u_char *
 		lntag,		/* libnet context pointer */
 		0		/* packet id */
 	) == -1 ) {
-		syslog(LOG_ERR, "libnet_build_arp failed:");
+		cl_log(LOG_ERR, "libnet_build_arp failed:");
 		return -1;
 	}
 
 	/* Ethernet header */
-	if (get_hw_addr(device, device_mac) < 0) {
-		syslog(LOG_ERR, "Cannot find mac address for %s",
+	if (get_hw_addr((char *)device, device_mac) < 0) {
+		cl_log(LOG_ERR, "Cannot find mac address for %s",
 				device);
 		return -1;
 	}
 
 	if (libnet_build_ethernet(bcast_mac, device_mac, ETHERTYPE_ARP, NULL, 0
 	,	lntag, 0) == -1 ) {
-		syslog(LOG_ERR, "libnet_build_ethernet failed:");
+		cl_log(LOG_ERR, "libnet_build_ethernet failed:");
 		return -1;
 	}
 
 	n = libnet_write(lntag);
 	if (n == -1) {
-		syslog(LOG_ERR, "libnet_build_ethernet failed:");
+		cl_log(LOG_ERR, "libnet_build_ethernet failed:");
 	}
 	libnet_clear_packet(lntag);
 
@@ -494,7 +533,7 @@ create_pid_directory(const char *pidfilename)
 
 	pidfilename_cpy = strdup(pidfilename);
 	if (!pidfilename_cpy) {
-		syslog(LOG_INFO, "Memory allocation failure: %s\n",
+		cl_log(LOG_INFO, "Memory allocation failure: %s\n",
 				strerror(errno));
 		return -1;
 	}
@@ -504,24 +543,29 @@ create_pid_directory(const char *pidfilename)
 	status = stat(dir, &stat_buf); 
 
 	if (status < 0 && errno != ENOENT && errno != ENOTDIR) {
-		syslog(LOG_INFO, "Could not stat pid-file directory "
+		cl_log(LOG_INFO, "Could not stat pid-file directory "
 				"[%s]: %s", dir, strerror(errno));
 		free(pidfilename_cpy);
 		return -1;
 	}
 	
-	if (!status) {
+	if (status >= 0) {
 		if (S_ISDIR(stat_buf.st_mode)) {
 			return 0;
 		}
-		syslog(LOG_INFO, "Pid-File directory exists but is "
+		cl_log(LOG_INFO, "Pid-File directory exists but is "
 				"not a directory [%s]", dir);
 		free(pidfilename_cpy);
 		return -1;
         }
 
 	if (mkdir(dir, S_IRUSR|S_IWUSR|S_IXUSR | S_IRGRP|S_IXGRP) < 0) {
-		syslog(LOG_INFO, "Could not create pid-file directory "
+		/* Did someone else make it while we were trying ? */
+		if (errno == EEXIST && stat(dir, &stat_buf) >= 0
+		&&	S_ISDIR(stat_buf.st_mode)) {
+			return 0;
+		}
+		cl_log(LOG_INFO, "Could not create pid-file directory "
 				"[%s]: %s", dir, strerror(errno));
 		free(pidfilename_cpy);
 		return -1;
@@ -542,7 +586,7 @@ write_pid_file(const char *pidfilename)
 	ssize_t 	bytes;
 
 	if (*pidfilename != '/') {
-		syslog(LOG_INFO, "Invalid pid-file name, must begin with a "
+		cl_log(LOG_INFO, "Invalid pid-file name, must begin with a "
 				"'/' [%s]\n", pidfilename);
 		return -1;
 	}
@@ -556,7 +600,7 @@ write_pid_file(const char *pidfilename)
 				S_IRUSR|S_IWUSR);
 		if (pidfilefd < 0) {
 			if (errno != EEXIST) { /* Old PID file */
-				syslog(LOG_INFO, "Could not open pid-file "
+				cl_log(LOG_INFO, "Could not open pid-file "
 						"[%s]: %s", pidfilename, 
 						strerror(errno));
 				return -1;
@@ -568,7 +612,7 @@ write_pid_file(const char *pidfilename)
 
 		pidfilefd = open(pidfilename, O_RDONLY, S_IRUSR|S_IWUSR);
 		if (pidfilefd < 0) {
-			syslog(LOG_INFO, "Could not open pid-file " 
+			cl_log(LOG_INFO, "Could not open pid-file " 
 					"[%s]: %s", pidfilename, 
 					strerror(errno));
 			return -1;
@@ -580,7 +624,7 @@ write_pid_file(const char *pidfilename)
 				if (errno == EINTR) {
 					continue;
 				}
-				syslog(LOG_INFO, "Could not read pid-file " 
+				cl_log(LOG_INFO, "Could not read pid-file " 
 						"[%s]: %s", pidfilename, 
 						strerror(errno));
 				return -1;
@@ -590,14 +634,14 @@ write_pid_file(const char *pidfilename)
 		}
 
 		if(unlink(pidfilename) < 0) {
-			syslog(LOG_INFO, "Could not delete pid-file "
+			cl_log(LOG_INFO, "Could not delete pid-file "
 	 				"[%s]: %s", pidfilename, 
 					strerror(errno));
 			return -1;
 		}
 
 		if (!bytes) {
-			syslog(LOG_INFO, "Invalid pid in pid-file "
+			cl_log(LOG_INFO, "Invalid pid in pid-file "
 	 				"[%s]: %s", pidfilename, 
 					strerror(errno));
 			return -1;
@@ -607,36 +651,36 @@ write_pid_file(const char *pidfilename)
 
 		pid = strtoul(pidbuf, NULL, 10);
 		if (pid == ULONG_MAX && errno == ERANGE) {
-			syslog(LOG_INFO, "Invalid pid in pid-file "
+			cl_log(LOG_INFO, "Invalid pid in pid-file "
 	 				"[%s]: %s", pidfilename, 
 					strerror(errno));
 			return -1;
 		}
 
 		if (kill(pid, SIGKILL) < 0 && errno != ESRCH) {
-			syslog(LOG_INFO, "Error killing old proccess [%lu] "
+			cl_log(LOG_INFO, "Error killing old proccess [%lu] "
 	 				"from pid-file [%s]: %s", pid,
 					pidfilename, strerror(errno));
 			return -1;
 		}
 
-		syslog(LOG_INFO, "Killed old send_arp process [%lu]\n",
+		cl_log(LOG_INFO, "Killed old send_arp process [%lu]\n",
 				pid);
 	}
 
 	if (snprintf(pidbuf, sizeof(pidbuf), "%u"
 	,	getpid()) >= (int)sizeof(pidbuf)) {
-		syslog(LOG_INFO, "Pid too long for buffer [%u]", getpid());
+		cl_log(LOG_INFO, "Pid too long for buffer [%u]", getpid());
 		return -1;
 	}
 
 	while (1) {
 		bytes = write(pidfilefd, pidbuf, strlen(pidbuf));
-		if (bytes != strlen(pidbuf)) {
+		if (bytes != (ssize_t)strlen(pidbuf)) {
 			if (bytes < 0 && errno == EINTR) {
 				continue;
 			}
-			syslog(LOG_INFO, "Could not write pid-file "
+			cl_log(LOG_INFO, "Could not write pid-file "
 					"[%s]: %s", pidfilename,
 					strerror(errno));
 			return -1;
@@ -650,163 +694,3 @@ write_pid_file(const char *pidfilename)
 }
 
 
-/*
- * $Log: send_arp.c,v $
- * Revision 1.7.2.3  2004/09/14 15:42:32  alan
- * BEAM FIX: size_t should have been ssize_t for read return code.
- *
- * Revision 1.7.2.2  2004/06/15 01:59:06  horms
- * Merged in changes from Head Branch
- *   1.12: Changes to make gratuitous ARP Packets RFC2002 (4.6) compliant
- *   1.11: Slightly better error checking
- *   1.10:  Fix for pid code for FreeBSD (and others)
- *
- * Revision 1.7.2.1  2004/04/20 05:05:29  alan
- * Backported everything from 1.3.0 to 1.2.1
- *
- * Revision 1.9  2004/03/19 15:07:43  alan
- * Put in a fix provided by Michael Dipper <md@LF.net>
- * A '/' is missing from the pidfile pathname for send_arp.
- *
- * Revision 1.8  2004/02/17 22:11:57  lars
- * Pet peeve removal: _Id et al now gone, replaced with consistent Id header.
- *
- * Revision 1.7  2004/01/22 01:53:35  alan
- * Changed send_arp to exit(0) when killed by SIGTERM.
- *
- * Revision 1.6  2003/12/11 22:58:04  alan
- * Put in a patch from Werner Schulthei for allowing old-style MAC addrs
- * in send_arp.
- *
- * Fixed some signed/unsigned warning issues in the code.
- *
- * Revision 1.5  2003/10/31 16:15:20  msoffen
- * Added limit.h to define MAXINT value.
- *
- * Revision 1.4  2003/10/29 11:40:50  horms
- * Send arp  creates a driectory to store its pid file in on-the-fly. IPaddr does likewise for state files.
- *
- * Revision 1.3  2003/10/23 07:36:29  horms
- * added pid file to sendarp
- *
- * Revision 1.2  2003/10/15 17:03:23  horms
- * Merged  get_hw_addr into and send_arp
- *
- * Revision 1.1  2003/10/15 09:52:12  horms
- * Move utilities that link against libnet (sendarp and get_hw_addr)
- * to their own directory so we no longer have to come up with
- * creative ways to selectively mangle CFLAGS.
- *
- * Revision 1.24  2003/09/26 06:02:13  alan
- * Fixed an undefined variable warning.
- *
- * Revision 1.23  2003/07/12 17:02:59  alan
- * Hopefully last fix for the changes made to allow user to specify arp intervals, etc.
- *
- * Revision 1.22  2003/07/12 16:19:54  alan
- * Fixed a bug in the new send_arp options and their invocation...
- *
- * Revision 1.21  2003/04/15 18:56:33  msoffen
- * Removed printf("\n") that served no purpose anymore (used to print .\n).
- *
- * Revision 1.20  2003/04/15 11:07:05  horms
- * errors go to stderr, not stdout
- *
- * Revision 1.19  2003/03/21 17:38:31  alan
- * Put in a patch by Thiago Rondon <thiago@nl.linux.org> to fix a minor
- * compile error in send_arp.c, which only affects the 1.1 libnet API code.
- *
- * Revision 1.18  2003/02/17 18:51:03  alan
- * Minor typo correction for #error line in send_arp.c
- *
- * Revision 1.17  2003/02/17 16:30:46  msoffen
- * Made it error out if libnet isn't defined at all (no 1.0 or 1.1 version).
- *
- * Revision 1.16  2003/02/17 15:31:50  alan
- * Fixed a nasty bug where we don't pass the interface to the libnet libraries
- * correctly.  Thanks to Steve Snodgrass for finding it.
- *
- * Revision 1.15  2003/02/07 08:37:17  horms
- * Removed inclusion of portability.h from .h files
- * so that it does not need to be installed.
- *
- * Revision 1.14  2003/02/05 09:06:33  horms
- * Lars put a lot of work into making sure that portability.h
- * is included first, everywhere. However this broke a few
- * things when building against heartbeat headers that
- * have been installed (usually somewhere under /usr/include or
- * /usr/local/include).
- *
- * This patch should resolve this problem without undoing all of
- * Lars's hard work.
- *
- * As an asside: I think that portability.h is a virus that has
- * infected all of heartbeat's code and now must also infect all
- * code that builds against heartbeat. I wish that it didn't need
- * to be included all over the place. Especially in headers to
- * be installed on the system. However, I respect Lars's opinion
- * that this is the best way to resolve some weird build problems
- * in the current tree.
- *
- * Revision 1.13  2003/01/31 10:02:09  lars
- * Various small code cleanups:
- * - Lots of "signed vs unsigned" comparison fixes
- * - time_t globally replaced with TIME_T
- * - All seqnos moved to "seqno_t", which defaults to unsigned long
- * - DIMOF() definition centralized to portability.h and typecast to int
- * - EOS define moved to portability.h
- * - dropped inclusion of signal.h from stonith.h, so that sigignore is
- *   properly defined
- *
- * Revision 1.12  2002/09/12 14:06:18  msoffen
- * Removed a write to stderr of ".", really served no purpose and always ran.
- * It was a carryover from the old send_arp.c.
- *
- * Revision 1.11  2002/09/05 06:12:42  alan
- * Put in code to recover a bug fix from Japan, plus
- * make the code hopefully work with both the old and new libnet APIs.
- *
- * Revision 1.10  2002/08/16 14:18:41  msoffen
- * Changes to get IP takeover working properly on OpenBSD
- * Changed how *BSD deletes an alias.
- * Create get_hw_addr (uses libnet package).
- *
- * Revision 1.9  2002/07/08 04:14:12  alan
- * Updated comments in the front of various files.
- * Removed Matt's Solaris fix (which seems to be illegal on Linux).
- *
- * Revision 1.8  2002/06/06 04:43:40  alan
- * Got rid of a warning (error) about an unused RCS version string.
- *
- * Revision 1.7  2002/05/28 18:25:48  msoffen
- * Changes to replace send_arp with a libnet based version.  This works accross
- * all operating systems we currently "support" (Linux, FreeBSD, Solaris).
- *
- * Revision 1.6  2001/10/24 00:21:58  alan
- * Fix to send both a broadcast ARP request as well as the ARP response we
- * were already sending.  All the interesting research work for this fix was
- * done by Masaki Hasegawa <masaki-h@pp.iij4u.or.jp> and his colleagues.
- *
- * Revision 1.5  2001/06/07 21:29:44  alan
- * Put in various portability changes to compile on Solaris w/o warnings.
- * The symptoms came courtesy of David Lee.
- *
- * Revision 1.4  2000/12/04 20:33:17  alan
- * OpenBSD fixes from Frank DENIS aka Jedi/Sector One <j@c9x.org>
- *
- * Revision 1.3  1999/10/05 06:17:29  alanr
- * Fixed various uninitialized variables
- *
- * Revision 1.2  1999/09/30 18:34:27  alanr
- * Matt Soffen's FreeBSD changes
- *
- * Revision 1.1.1.1  1999/09/23 15:31:24  alanr
- * High-Availability Linux
- *
- * Revision 1.4  1999/09/08 03:46:27  alanr
- * Changed things so they work when rearranged to match the FHS :-)
- *
- * Revision 1.3  1999/08/17 03:49:09  alanr
- * *** empty log message ***
- *
- */
