@@ -1,4 +1,3 @@
-/* $Id: ipc.h,v 1.30.2.2 2004/12/08 19:14:47 gshi Exp $ */
 /*
  * ipc.h IPC abstraction data structures.
  *
@@ -33,18 +32,16 @@
 #include <sys/types.h>
 #include <sys/poll.h>
 
-#ifdef HAVE_STRING_H
-#include <string.h>
-#endif
+#ifdef IPC_TIME_DEBUG
+#include <clplumbing/longclock.h>
+#define MAXIPCTIME 3000
 
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
 #endif
 
 /* constants */
-#define DEFAULT_MAX_QLEN 20
+#define DEFAULT_MAX_QLEN 64
 #define MAX_MESSAGE_SIZE 4096
-
+#define MAX_MSGPAD 128
 /* channel and connection status */
 #define IPC_CONNECT		1	/* Connected: can read, write */
 #define IPC_WAIT		2	/* Waiting for connection */
@@ -52,6 +49,12 @@
 #define IPC_DISC_PENDING	4	/* Disconnected, can't write but    */
 					/* may be more data to read	    */
 
+#define MAXFAILREASON		128
+
+#define IPC_SERVER		1
+#define IPC_CLIENT		2
+#define IPC_PEER		3
+
 #define IPC_ISRCONN(ch) ((ch)->ch_status == IPC_CONNECT		\
 	||	(ch)->ch_status == IPC_DISC_PENDING)
 
@@ -62,7 +65,6 @@
 #define IPC_FAIL 1
 #define IPC_BROKEN 2
 #define IPC_INTR 3
-
 /*
  *	IPC:  Sockets-like Interprocess Communication Abstraction
  *
@@ -111,6 +113,8 @@ typedef struct IPC_AUTH			IPC_Auth;
 typedef struct IPC_OPS			IPC_Ops;
 typedef struct IPC_WAIT_OPS		IPC_WaitOps;
 
+
+
 /* wait connection structure. */
 struct IPC_WAIT_CONNECTION{
 	int		ch_status;	/* wait conn. status.*/
@@ -118,6 +122,9 @@ struct IPC_WAIT_CONNECTION{
 	IPC_WaitOps	*ops;		/* wait conn. function table .*/
 };
 
+
+typedef void(*flow_callback_t)(IPC_Channel*, void*);
+
 /* channel structure.*/
 struct IPC_CHANNEL{
 	int		ch_status;	/* identify the status of channel.*/
@@ -125,36 +132,52 @@ struct IPC_CHANNEL{
 	void*		ch_private;	/* channel private data. */
 					/* (may contain conn. info.) */
 	IPC_Ops*	ops;		/* IPC_Channel function table.*/
-	
+
+	/* number of bytes needed
+	 * at the begginging of <ipcmessage>->msg_body
+	 * it's for msg head needed to tranmit in wire	 
+	 */
+	unsigned int	msgpad;
 	
 	/* the number of bytes remainng to send for the first message in send queue
-           0 means nothing has been sent thus all bytes needs to be send
-           n != 0 means there are still n bytes needs to be sent
-        */
-        unsigned int    bytes_remaining;
-	
-	
-        /* is the send blocking or nonblocking*/
-        gboolean        is_send_blocking;
-	
+	   0 means nothing has been sent thus all bytes needs to be send
+	   n != 0 means there are still n bytes needs to be sent
+	*/
+	unsigned int	bytes_remaining;
 
 
-/*
- *  There are two queues in channel. One is for sending and the other
+	/* is the send blocking or nonblocking*/
+	gboolean	should_send_block;
+	
+/*  There are two queues in channel. One is for sending and the other
  *  is for receiving. 
  *  Those two queues are channel's internal queues. They should not be 
  *  accessed directly.
  */
-/* private: */
+	/* private: */
 	IPC_Queue*	send_queue; 
 	IPC_Queue*	recv_queue; 
 
+	/* buffer pool for receive in this channel*/
+	struct ipc_bufpool* pool;
+
+	/* the follwing is for send flow control*/
+	int		high_flow_mark;
+	int		low_flow_mark;
+	void*		high_flow_userdata;
+	void*		low_flow_userdata;
+	flow_callback_t	high_flow_callback;
+	flow_callback_t	low_flow_callback;
+	
+	int		conntype;
+	
+	char		failreason[MAXFAILREASON];
 };
 
 struct IPC_QUEUE{
-	int		current_qlen;	/* Current qlen */
-	int		max_qlen;	/* Max allowed qlen */
-	GList*	queue;		/* List of messages */
+	size_t		current_qlen;	/* Current qlen */
+	size_t		max_qlen;	/* Max allowed qlen */
+	GList*		queue;		/* List of messages */
 };
 
 /* authentication information : set of gids and uids */
@@ -163,9 +186,11 @@ struct IPC_AUTH {
 	GHashTable * gid;	/* hash table for group id */
 };
 
+
 /* Message structure. */
 struct IPC_MESSAGE{
 	size_t	msg_len;
+	void*	msg_buf;
 	void*	msg_body;
 /* 
  * IPC_MESSAGE::msg_done 
@@ -183,6 +208,7 @@ struct IPC_MESSAGE{
 				/* May be used by callback function. */
 	IPC_Channel * msg_ch;	/* Channel the */
 				/* message is from/in */
+
 };
 
 struct IPC_WAIT_OPS{
@@ -485,6 +511,75 @@ struct IPC_OPS{
  *
  */
 	int  (* set_recv_qlen) (IPC_Channel * ch, int q_len);
+
+
+/*
+ * IPC_OPS: set callback for high/low flow mark
+ * ch	(IN) : the pointer to the channel
+ * callback (IN) : the callback function
+ * user_data(IN) : a pointer to user_data
+ *		   callback will be called with channel and
+ *		   this user_data as parameters
+ *
+ * Return values:
+ *	void
+ *
+ */
+	
+	
+	void (* set_high_flow_callback) (IPC_Channel* ch , 	
+					 flow_callback_t callback,
+					 void* user_data);
+	
+	void (* set_low_flow_callback) (IPC_Channel* ch , 	
+					 flow_callback_t callback,
+					 void* user_data);
+	
+/*
+ * IPC_OPS::new_ipcmsg
+ * ch	(IN) : the pointer to the channel
+ * data (IN) : data to be copied to the message body
+ * len	(IN) : data len
+ * private (IN): the pointer value to set as in the message
+ *
+ * Return values:
+ *	the pointer to a new created message will be
+ *	returned if success or NULL if failure
+ *
+ */
+	
+	IPC_Message*	(*new_ipcmsg)(IPC_Channel* ch, const void* data, 
+				      int len, void* private);
+	
+	
+/*
+ * IPC_OPS::nget_chan_status
+ * ch	(IN) : the pointer to the channel
+ *
+ * Return value:
+ *	channel status.
+ *
+ */
+	int	(*get_chan_status)(IPC_Channel* ch);
+
+	
+/*
+ * These two functions returns true if the corresponding queue 
+ * is full, otherwise it returns false
+ */
+	
+	gboolean (*is_sendq_full)(struct IPC_CHANNEL * ch);
+	gboolean (*is_recvq_full)(struct IPC_CHANNEL * ch);
+
+
+	/* Get the connection type for the channel
+	 * it can be IPC_SERVER, IPC_CLIENT, IPC_PEER
+	 */
+	
+	int (*get_conntype)(struct IPC_CHANNEL* ch);
+
+	int (*disconnect)(struct IPC_CHANNEL* ch);
+		
 };
 
 
@@ -566,14 +661,83 @@ int ipc_channel_pair(IPC_Channel* channels[2]);
  *	be created.
  *
  */
+
+
+IPC_Auth*  ipc_str_to_auth(const char * uidlist, int, const char * gidlist, int);
+
 extern IPC_Auth * ipc_set_auth(uid_t * a_uid, gid_t * a_gid
 ,	int num_uid, int num_gid);
 
-/* Destroys an object constructed by ipc_set_auth */
+/* Destroys an object constructed by ipc_set_auth or ipc_str_to_auth() */
 extern void ipc_destroy_auth(IPC_Auth * auth);
 
 extern void ipc_set_pollfunc(int (*)(struct pollfd*, unsigned int, int));
 
+
+#ifdef IPC_TIME_DEBUG
+
+enum MSGPOS_IN_IPC{
+	MSGPOS_ENQUEUE,
+	MSGPOS_SEND,
+	MSGPOS_RECV,
+	MSGPOS_DEQUEUE
+};
+
+#endif
+
+
+struct SOCKET_MSG_HEAD{
+	int msg_len;
+	unsigned int magic;
+#ifdef IPC_TIME_DEBUG
+	longclock_t enqueue_time;
+	longclock_t send_time;
+	longclock_t recv_time;
+	longclock_t dequeue_time;
+#endif
+
+};
+
+
+/* MAXMSG is the maximum final message size on the wire. */
+#define	MAXMSG		(256*1024)
+/* MAXUNCOMPRESSED is the maximum, raw data size prior to compression. */
+#define	MAXUNCOMPRESSED	(2048*1024)
+#define HEADMAGIC	0xabcd
+#define POOL_SIZE (4*1024)
+struct ipc_bufpool{
+	
+	int refcount;
+	char* currpos;
+	char* consumepos;
+	char* startpos;
+	char* endpos;
+	int size;
+};
+
+struct ipc_bufpool* ipc_bufpool_new(int);
+
+void	ipc_bufpool_del(struct ipc_bufpool* pool);
+
+int	ipc_bufpool_spaceleft(struct ipc_bufpool* pool);
+
+int	ipc_bufpool_update(struct ipc_bufpool* pool,
+			   struct IPC_CHANNEL * ch,
+			   int msg_len,
+			   IPC_Queue* rqueue);
+
+gboolean	ipc_bufpool_full(struct ipc_bufpool* pool,
+				 struct IPC_CHANNEL* ch,
+				 int*);
+int		ipc_bufpool_partial_copy(struct ipc_bufpool* dstpool,
+					 struct ipc_bufpool* srcpool);
+
+void	ipc_bufpool_ref(struct ipc_bufpool* pool);
+
+void	ipc_bufpool_unref(struct ipc_bufpool* pool);
+
+void	set_ipc_time_debug_flag(gboolean flag);
+
 #define	IPC_PATH_ATTR		"path"		/* pathname attribute */
 #define	IPC_DOMAIN_SOCKET	"uds"		/* Unix domain socket */
 #define IPC_MODE_ATTR           "sockmode"      /* socket mode attribute */
