@@ -1,4 +1,19 @@
-/* $Id: realtime.h,v 1.5.2.1 2004/04/20 07:57:54 alan Exp $ */
+/*
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ * 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
 #ifndef _CLPLUMBING_REALTIME_H
 #	define _CLPLUMBING_REALTIME_H
 #	include <sched.h>
@@ -17,7 +32,7 @@
  */
 
 void cl_make_realtime
-(	int spolicy,	/* SCHED_RR or SCHED_FIFO */
+(	int spolicy,	/* SCHED_RR or SCHED_FIFO (or SCHED_OTHER) */
 	int priority,	/* typically 1-99 */
 	int stackgrowK,	/* Amount to grow stack by */
 	int heapgrowK	/* Amount to grow heap by */
@@ -40,5 +55,7 @@ void cl_realtime_malloc_check(void);
 
 /* Number of times we "go to the well" for memory after becoming realtime */
 int cl_nonrealtime_malloc_count(void);
+/* Number of bytes we "got from the well" for memory after becoming realtime */
+unsigned long cl_nonrealtime_malloc_size(void);
 
 #endif
