@@ -1,5 +1,4 @@
-/* $Id: cl_poll.c,v 1.20.2.1 2004/04/20 08:39:45 alan Exp $ */
-#include <portability.h>
+#include <lha_internal.h>
 #include <stdlib.h>
 #include <unistd.h>
 /*
@@ -198,7 +197,7 @@ cl_poll_prepsig(int nsig)
 	if (debug) {
 		cl_log(LOG_DEBUG
 		,	"Signal %d belongs to us...", nsig);
-		cl_log(LOG_DEBUG, "cl_poll_prepsig(%d) succeeded.\n", nsig);
+		cl_log(LOG_DEBUG, "cl_poll_prepsig(%d) succeeded.", nsig);
 	}
 	
 	return 0;
@@ -481,7 +480,7 @@ cl_poll_sigaction(int nsig, siginfo_t* info, void* v)
 
 	/* We should not call logging functions in (real) signal handlers */
 	if (nsig != monitorinfo[fd].nsig) {
-		cl_log(LOG_ERR, "cl_poll_sigaction called with signal %d/%d\n"
+		cl_log(LOG_ERR, "cl_poll_sigaction called with signal %d/%d"
 		,	nsig, monitorinfo[fd].nsig);
 	}
 
