@@ -1,4 +1,3 @@
-/* $Id: pils.c,v 1.34.2.2 2004/09/06 22:21:38 gshi Exp $ */
 /*
  * Copyright (C) 2001 Alan Robertson <alanr@unix.sh>
  * This software licensed under the GNU LGPL.
@@ -19,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-#include <portability.h>
+#include <lha_internal.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -40,6 +39,10 @@
 #define ENABLE_PIL_DEFS_PRIVATE
 #define ENABLE_PLUGIN_MANAGER_PRIVATE
 
+#ifndef STRLEN_CONST
+#	define STRLEN_CONST(con)	(sizeof(con)-1)
+#endif
+
 #include <pils/interface.h>
 
 #define NEW(type)	(g_new(type,1))
@@ -212,11 +215,11 @@ PILRegisterInterface
 (	PILPlugin*	piinfo
 ,	const char *	interfacetype	/* Type of interface		*/
 ,	const char *	interfacename	/* Name of interface		*/
-,	void*		Ops		/* Ops exported by this interface	*/
-,	PILInterfaceFun	closefunc	/* Ops exported by this interface	*/
-,	PILInterface**	interfaceid	/* Interface id 	(OP)		*/
-,	void**		Imports		/* Functions imported by
-					 this interface	(OP)		*/
+,	void*		Ops		/* Ops exported by this interface */
+,	PILInterfaceFun	closefunc	/* Ops exported by this interface */
+,	PILInterface**	interfaceid	/* Interface id 	(OP)	*/
+,	void**		Imports		/* Functions imported by	*/
+					/* this interface	(OP)	*/
 ,	void*		ud_interface	/* interface user data 		*/
 );
 static PIL_rc	PILunregister_interface(PILInterface* interfaceid);
@@ -234,10 +237,11 @@ static PILPluginImports PILPluginImportSet =
 {	PILregister_plugin	/* register_plugin */
 ,	PILunregister_plugin	/* unregister_plugin */
 ,	PILRegisterInterface	/* register_interface */
-,	RemoveAPILInterface /* unregister_interface */
+,	RemoveAPILInterface	/* unregister_interface */
 ,	PILLoadPlugin		/* load_plugin */
 ,	PILLog			/* Logging function */
 ,	g_malloc		/* Malloc function */
+,	g_realloc		/* realloc function */
 ,	g_free			/* Free function */
 ,	g_strdup		/* Strdup function */
 };
@@ -485,6 +489,29 @@ NewPILPluginUniv(const char * basepluginpath)
 	return ret;
 }
 
+/* Change memory allocation functions immediately  after creating universe */
+void
+PilPluginUnivSetMemalloc(PILPluginUniv* u
+,	void*	(*allocfun)(unsigned long size)
+,	void*	(*reallocfun)(void * ptr, unsigned long size)
+,	void	(*freefun)(void* space)
+,	char*	(*strdupfun)(const char *s))
+{
+	u->imports->alloc	= allocfun;
+	u->imports->mrealloc	= reallocfun;
+	u->imports->mfree	= freefun;
+	u->imports->mstrdup	= strdupfun;
+}
+
+
+/* Change logging functions - preferably right after creating universe */
+void
+PilPluginUnivSetLog(PILPluginUniv* u
+,	void	(*logfun)	(PILLogLevel priority, const char * fmt, ...))
+{
+	u->imports->log	= logfun;
+}
+
 void
 DelPILPluginUniv(PILPluginUniv* piuniv)
 {
@@ -557,7 +584,7 @@ RemoveAPILPluginType(PILPluginType*Plugintype)
  *	InterfaceManager_plugin_init: Initialize the handling of
  *	"Interface Manager" interfaces.
  *
- *	There are a few potential bootstraiffng problems here ;-)
+ *	There are a few potential bootstrapping problems here ;-)
  *
  */
 static PIL_rc
@@ -688,8 +715,9 @@ PILSetDebugLevel(PILPluginUniv* u, const char * pitype, const char * piname
 {
 	struct set_debug_helper helper = {pitype, piname, level};
 
-	if (u == NULL)
+	if (u == NULL) {
 		return;
+	}
 
 	if (pitype == NULL) {
 		g_hash_table_foreach(u->PluginTypes
@@ -767,12 +795,12 @@ NewPILInterface(PILInterfaceType*	interfacetype
 	}
 	ret = NEW(PILInterface);
 	STATNEW(interface);
-	ret->MagicNum = PIL_MAGIC_INTERFACE;
 	if (DEBUGPLUGIN) {
 		PILLog(PIL_DEBUG, "NewPILInterface(0x%x)", (unsigned long)ret);
 	}
 
 	if (ret) {
+		ret->MagicNum = PIL_MAGIC_INTERFACE;
 		ret->interfacetype = interfacetype;
 		ret->exports = exports;
 		ret->ud_interface = ud_interface;
@@ -1524,7 +1552,6 @@ PILRegisterInterface(PILPlugin* piinfo
 	||	(piuniv = pitype->piuniv)	== NULL
 	||	(ifuniv = piuniv->ifuniv)	== NULL
 	||	ifuniv->iftypes			== NULL
-	||	close_func			== NULL
 	) {
 		REPORTERR("bad parameters to PILRegisterInterface");
 		return PIL_INVAL;
@@ -1621,8 +1648,7 @@ PILunregister_interface(PILInterface* id)
 	if (	 id == NULL
 	||	(t = id->interfacetype) == NULL
 	||	(u = t->universe) == NULL
-	|| 	id->interfacename == NULL
-	||	id->if_close == NULL) {
+	|| 	id->interfacename == NULL) {
 		PILLog(PIL_WARN, "PILunregister_interface: bad interfaceid");
 		return PIL_INVAL;
 	}
@@ -1639,10 +1665,13 @@ PILunregister_interface(PILInterface* id)
 
 	/* Call the close function supplied by the interface */
 
-	if ((rc=id->if_close(id, id->ud_interface)) != PIL_OK) {
+	if ((id->if_close != NULL) 
+	&&	((rc=id->if_close(id, id->ud_interface)) != PIL_OK)) {
 		PILLog(PIL_WARN, "InterfaceClose on %s/%s returned %s"
 		,	t->typename, id->interfacename
 		,	PIL_strerror(rc));
+	} else {
+		rc = PIL_OK;
 	}
 
 	/* Find the InterfaceManager that manages us */
@@ -1834,13 +1863,13 @@ so_select (const struct dirent *dire)
     
 	const char obj_end [] = PLUGINSUFFIX;
 	const char *end = &dire->d_name[strlen(dire->d_name)
-	-	(STRLEN(obj_end))];
+	-	(STRLEN_CONST(obj_end))];
 	
 	
 	if (DEBUGPLUGIN) {
 		PILLog(PIL_DEBUG, "In so_select: %s.", dire->d_name);
 	}
-	if (obj_end < dire->d_name) {
+	if (end < dire->d_name) {
 			return 0;
 	}
 	if (strcmp(end, obj_end) == 0) {
@@ -1857,7 +1886,7 @@ so_select (const struct dirent *dire)
 		,	dire->d_name, end
 		,	sizeof(obj_end), strlen(dire->d_name)
 		,	&dire->d_name[strlen(dire->d_name)
-		-	(sizeof(obj_end)-1)]);
+		-	(STRLEN_CONST(obj_end))]);
 	}
 	
 	return 0;
@@ -1893,11 +1922,15 @@ PILPluginTypeListPlugins(PILPluginType* pitype
 			}
 		}
 
-
 		files = NULL;
 		errno = 0;
 		dircount = scandir(path->str, &files
 		,	SCANSEL_CAST so_select, NULL);
+		if (DEBUGPLUGIN) {
+			PILLog(PIL_DEBUG, "PILS: Examining directory [%s]"
+			": [%d] files matching [%s] suffix found."
+			,	path->str, dircount, PLUGINSUFFIX);
+		}
 		g_string_free(path, 1); path=NULL;
 
 		if (dircount <= 0) {
@@ -1905,6 +1938,11 @@ PILPluginTypeListPlugins(PILPluginType* pitype
 				FREE_DIRLIST(files, dircount);
 				files = NULL;
 			}
+			if (DEBUGPLUGIN) {
+				PILLog(PIL_DEBUG
+				,	"PILS: skipping empty directory"
+				" in PILPluginTypeListPlugins()");
+			}
 			continue;
 		}
 
@@ -1920,12 +1958,16 @@ PILPluginTypeListPlugins(PILPluginType* pitype
 		for (j=0; j < dircount; ++j) {
 			char*	s;
 			unsigned slen = strlen(files[j]->d_name)
-			-	STRLEN(PLUGINSUFFIX);
+			-	STRLEN_CONST(PLUGINSUFFIX);
 
 			s = g_malloc(slen+1);
 			strncpy(s, files[j]->d_name, slen);
 			s[slen] = EOS;
 			result[initoff+j] = s;
+			if (DEBUGPLUGIN) {
+				PILLog(PIL_DEBUG, "PILS: plugin [%s] found"
+				,	s);
+			}
 		}
 		FREE_DIRLIST(files, dircount);
 		files = NULL;
@@ -1938,6 +1980,11 @@ PILPluginTypeListPlugins(PILPluginType* pitype
 		result[plugincount] = NULL;
 		/* Return them in sorted order... */
 		qsort(result, plugincount, sizeof(char *), qsort_string_cmp);
+	}else{
+		if (DEBUGPLUGIN) {
+			PILLog(PIL_DEBUG, "PILS: NULL return"
+			" from PILPluginTypeListPlugins()");
+		}
 	}
 
 
@@ -2038,7 +2085,6 @@ PILValidateInterface(gpointer key, gpointer interface, gpointer iftype)
 	g_assert(strcmp(Interface->interfacetype->typename
 	,	Interface->ifmanager->interfacename)== 0);
 	g_assert(Interface->exports != NULL);
-	g_assert(Interface->if_close != NULL);
 }
 static void
 PILValidateInterfaceType(gpointer key, gpointer iftype, gpointer ifuniv)
@@ -2094,10 +2140,12 @@ PILCallLog(PILLogFun logfun, PILLogLevel priority, const char * fmt, ...)
 {
 	va_list		args;
 	char *		str;
+	int		err = errno;
 
 	va_start (args, fmt);
 	str = g_strdup_vprintf(fmt, args);
 	va_end (args);
 	logfun(priority, "%s", str);
 	g_free(str);
+	errno = err;
 }
