@@ -1,4 +1,3 @@
-/* $Id: ping.c,v 1.26.2.6 2005/08/10 04:08:17 horms Exp $ */
 /*
  * ping.c: ICMP-echo-based heartbeat code for heartbeat.
  *
@@ -29,9 +28,10 @@
  *
  */
 
-#include <portability.h>
+#include <lha_internal.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stddef.h>
 #include <unistd.h>
 #include <errno.h>
 #include <string.h>
@@ -71,6 +71,7 @@
 #include <arpa/inet.h>
 
 #include <netdb.h>
+#include <clplumbing/uids.h>
 #include <heartbeat.h>
 #include <HBcomm.h>
 
@@ -115,31 +116,6 @@ static int		ping_isping(void);
 #define		ISPINGOBJECT(mp)	((mp) && ((mp)->vf == (void*)&pingOps))
 #define		PINGASSERT(mp)	g_assert(ISPINGOBJECT(mp))
 
-/*
- * pingclose is called as part of unloading the ping HBcomm plugin.
- * If there was any global data allocated, or file descriptors opened, etc.
- * which is associated with the plugin, and not a single interface
- * in particular, here's our chance to clean it up.
- */
-
-static void
-pingclosepi(PILPlugin*pi)
-{
-}
-
-
-/*
- * pingcloseintf called as part of shutting down the ping HBcomm interface.
- * If there was any global data allocated, or file descriptors opened, etc.
- * which is associated with the ping implementation, here's our chance
- * to clean it up.
- */
-static PIL_rc
-pingcloseintf(PILInterface* pi, void* pd)
-{
-	return PIL_OK;
-}
-
 static struct hb_media_fns pingOps ={
 	ping_new,	/* Create single object function */
 	NULL,		/* whole-line parse function */
@@ -152,7 +128,7 @@ static struct hb_media_fns pingOps ={
 	ping_isping,
 };
 
-PIL_PLUGIN_BOILERPLATE("1.0", Debug, pingclosepi);
+PIL_PLUGIN_BOILERPLATE2("1.0", Debug)
 
 static const PILPluginImports*  PluginImports;
 static PILPlugin*               OurPlugin;
@@ -184,7 +160,7 @@ PIL_PLUGIN_INIT(PILPlugin*us, const PILPluginImports* imports)
  	return imports->register_interface(us, PIL_PLUGINTYPE_S
 	,	PIL_PLUGIN_S
 	,	&pingOps
-	,	pingcloseintf		/*close */
+	,	NULL		/*close */
 	,	&OurInterface
 	,	(void*)&OurImports
 	,	interfprivate); 
@@ -287,7 +263,7 @@ ping_new(const char * host)
 }
 
 /*
- *	Close UDP/IP broadcast heartbeat interface
+ *	Close ICMP ping heartbeat interface
  */
 
 static int
@@ -311,11 +287,11 @@ ping_close(struct hb_media* mp)
 
 /*
  * Receive a heartbeat ping reply packet.
- * This code only needs to run once per instance of heartbeat.
- * Otherwise we receive "n" copies of every single ping packet.
+ * NOTE: This code only needs to run once for ALL ping nodes.
  * FIXME!!
  */
 
+static char ping_pkt[MAXLINE];
 static void *
 ping_read(struct hb_media* mp, int *lenp)
 {
@@ -334,7 +310,6 @@ ping_read(struct hb_media* mp, int *lenp)
 	int			hlen;
 	struct ha_msg *		msg;
 	const char 		*comment;
-	char			*pkt;
 	int			pktlen;
 	
 	PINGASSERT(mp);
@@ -384,36 +359,29 @@ ReRead:	/* We recv lots of packets that aren't ours */
 	}
 	
 	pktlen = numbytes - hlen - ICMP_HDR_SZ;
-	if ((pkt = ha_malloc(pktlen + 1)) == NULL) {
-		errno = ENOMEM;
-		return NULL;
-	}
-	pkt[pktlen] = 0;
-	
-	memcpy(pkt, buf.cbuf + hlen + ICMP_HDR_SZ, pktlen);	
+
+	memcpy(ping_pkt, buf.cbuf + hlen + ICMP_HDR_SZ, pktlen);
+	ping_pkt[pktlen] = 0;
 	*lenp = pktlen + 1;
- 
 	
-	msg = wirefmt2msg(msgstart, bufmax - msgstart);
+	msg = wirefmt2msg(msgstart, bufmax - msgstart, MSG_NEEDAUTH);
 	if (msg == NULL) {
-                ha_free(pkt);
 		errno = EINVAL;
 		return(NULL);
 	}
 	comment = ha_msg_value(msg, F_COMMENT);
-	if (comment == NULL || strcmp(comment, PIL_PLUGIN_S)) {
-                ha_free(pkt);
+	if (comment == NULL || strcmp(comment, PIL_PLUGIN_S) != 0) {
 		ha_msg_del(msg);
 		errno = EINVAL;
 		return(NULL);
 	}
 	
 	ha_msg_del(msg);
-	return(pkt);
+	return (ping_pkt);
 }
 
 /*
- * Send a heartbeat packet over broadcast UDP/IP interface
+ * Send a heartbeat packet over ICMP ping channel
  *
  * The peculiar thing here is that we don't send the packet we're given at all
  *
@@ -442,9 +410,10 @@ ping_write(struct hb_media* mp, void *p, int len)
 	const char *		ts;
 	struct ha_msg *		nmsg;
 	struct ha_msg *		msg;
+	static gboolean		needroot = FALSE;
 	
 	
-	msg = wirefmt2msg(p, len);
+	msg = wirefmt2msg(p, len, MSG_NEEDAUTH);
 	if( !msg){
 		PILCallLog(LOG, PIL_CRIT, "ping_write(): cannot convert wirefmt to msg");
 		return(HA_FAIL);
@@ -506,7 +475,7 @@ ping_write(struct hb_media* mp, void *p, int len)
 
 	if ((icmp_pkt = MALLOC(pktsize)) == NULL) {
 		PILCallLog(LOG, PIL_CRIT, "out of memory");
-		ha_free(pkt);
+		cl_free(pkt);
 		ha_msg_del(msg);
 		return HA_FAIL;
 	}
@@ -520,26 +489,39 @@ ping_write(struct hb_media* mp, void *p, int len)
 	++ei->iseq;
 
 	memcpy(icp->icmp_data, pkt, size);
-	ha_free(pkt); pkt = NULL;
+	cl_free(pkt); pkt = NULL;
 
 	/* Compute the ICMP checksum */
 	icp->icmp_cksum = in_cksum((u_short *)icp, pktsize);
 
+retry:
+	if (needroot) {
+		return_to_orig_privs();
+	}
+
 	if ((rc=sendto(ei->sock, (void *) icmp_pkt, pktsize, 0
 	,	(struct sockaddr *)&ei->addr
 	,	sizeof(struct sockaddr))) != (ssize_t)pktsize) {
+		if (errno == EPERM && !needroot) {
+			needroot=TRUE;
+			goto retry;
+		}
 		PILCallLog(LOG, PIL_CRIT, "Error sending packet: %s", strerror(errno));
 		FREE(icmp_pkt);
 		ha_msg_del(msg);
 		return(HA_FAIL);
 	}
+	if (needroot) {
+		return_to_dropped_privs();
+	}
 
 	if (DEBUGPKT) {
 		PILCallLog(LOG, PIL_DEBUG, "sent %d bytes to %s"
 		,	rc, inet_ntoa(ei->addr.sin_addr));
    	}
 	if (DEBUGPKTCONT) {
-		PILCallLog(LOG, PIL_DEBUG, "%s", (const char*)pkt);
+		PILCallLog(LOG, PIL_DEBUG, "ping pkt: %s"
+		,	icp->icmp_data);
    	}
 	FREE(icmp_pkt);
 	ha_msg_del(msg);
