@@ -1,4 +1,3 @@
-/* $Id: rps10.c,v 1.8.2.2 2004/09/14 16:43:46 alan Exp $ */
 /*
  *	Stonith module for WTI Remote Power Controllers (RPS-10M device)
  *
@@ -9,6 +8,8 @@
  *	Copyright (c) 2000 Computer Generation Incorporated
  *               Eric Z. Ayers <eric.ayers@compgen.com>
  *
+ * 	Mangled by Zhaokai <zhaokai@cn.ibm.com>, IBM, 2005
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -25,92 +26,49 @@
  *
  */
 
-#include <portability.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <string.h>
-#include <errno.h>
-#include <syslog.h>
-#include <libintl.h>
-#include <sys/wait.h>
+#define	DEVICE	"WTI RPS10 Power Switch"
+#include "stonith_plugin_common.h"
+
 #include <termios.h>
-#include <sys/time.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <glib.h>
-
-#include <stonith/stonith.h>
-
-#define PIL_PLUGINTYPE          STONITH_TYPE
-#define PIL_PLUGINTYPE_S        STONITH_TYPE_S
 #define PIL_PLUGIN              rps10
 #define PIL_PLUGIN_S            "rps10"
 #define PIL_PLUGINLICENSE 	LICENSE_LGPL
 #define PIL_PLUGINLICENSEURL 	URL_LGPL
+#define	ST_RPS10		"serial_to_targets"
+#define MAX_PRSID		256
 #include <pils/plugin.h>
 
-/*
- * rps10close is called as part of unloading the rps10 STONITH plugin.
- * If there was any global data allocated, or file descriptors opened, etc.
- * which is associated with the plugin, and not a single interface
- * in particular, here's our chance to clean it up.
- */
-
-static void
-rps10closepi(PILPlugin*pi)
-{
-}
-
-
-/*
- * rps10closeintf called as part of shutting down the rps10 STONITH
- * interface.  If there was any global data allocated, or file descriptors
- * opened, etc.  which is associated with the rps10 implementation,
- * here's our chance to clean it up.
- */
-static PIL_rc
-rps10closeintf(PILInterface* pi, void* pd)
-{
-	return PIL_OK;
-}
-
-static void *		rps10_new(void);
-static void		rps10_destroy(Stonith *);
-static int		rps10_set_config_file(Stonith *, const char * cfgname);
-static int		rps10_set_config_info(Stonith *, const char * info);
-static const char *	rps10_getinfo(Stonith * s, int InfoType);
-static int		rps10_status(Stonith * );
-static int		rps10_reset_req(Stonith * s, int request, const char * host);
-static char **		rps10_hostlist(Stonith  *);
-static void		rps10_free_hostlist(char **);
+static StonithPlugin *	rps10_new(const char *);
+static void		rps10_destroy(StonithPlugin *);
+static int		rps10_set_config(StonithPlugin *, StonithNVpair *);
+static const char**	rps10_get_confignames(StonithPlugin *);
+static const char *	rps10_getinfo(StonithPlugin * s, int InfoType);
+static int		rps10_status(StonithPlugin * );
+static int		rps10_reset_req(StonithPlugin * s, int request, const char * host);
+static char **		rps10_hostlist(StonithPlugin  *);
 
 static struct stonith_ops rps10Ops ={
-	rps10_new,		/* Create new STONITH object	*/
-	rps10_destroy,		/* Destroy STONITH object	*/
-	rps10_set_config_file,	/* set configuration from file	*/
-	rps10_set_config_info,	/* Get configuration from file	*/
-	rps10_getinfo,		/* Return STONITH info string	*/
-	rps10_status,		/* Return STONITH device status	*/
-	rps10_reset_req,		/* Request a reset */
-	rps10_hostlist,		/* Return list of supported hosts */
-	rps10_free_hostlist	/* free above list */
+	rps10_new,		/* Create new STONITH object		*/
+	rps10_destroy,		/* Destroy STONITH object		*/
+	rps10_getinfo,		/* Return STONITH info string		*/
+	rps10_get_confignames,	/* Return STONITH info string		*/
+	rps10_set_config,	/* Get configuration from NVpairs	*/
+	rps10_status,		/* Return STONITH device status		*/
+	rps10_reset_req,	/* Request a reset 			*/
+	rps10_hostlist,		/* Return list of supported hosts 	*/
 };
 
-PIL_PLUGIN_BOILERPLATE("1.0", Debug, rps10closepi);
+PIL_PLUGIN_BOILERPLATE2("1.0", Debug)
 static const PILPluginImports*  PluginImports;
 static PILPlugin*               OurPlugin;
 static PILInterface*		OurInterface;
 static StonithImports*		OurImports;
 static void*			interfprivate;
 
-#define LOG		PluginImports->log
-#define MALLOC		PluginImports->alloc
-#define STRDUP  	PluginImports->mstrdup
-#define FREE		PluginImports->mfree
-#define EXPECT_TOK	OurImports->ExpectToken
-#define STARTPROC	OurImports->StartProcess
+#include "stonith_signal.h"
+#define  DOESNT_USE_STONITHKILLCOMM
+#define  DOESNT_USE_STONITHSCANLINE
+#include "stonith_expect_helpers.h"
 
 PIL_rc
 PIL_PLUGIN_INIT(PILPlugin*us, const PILPluginImports* imports);
@@ -131,17 +89,12 @@ PIL_PLUGIN_INIT(PILPlugin*us, const PILPluginImports* imports)
  	return imports->register_interface(us, PIL_PLUGINTYPE_S
 	,	PIL_PLUGIN_S
 	,	&rps10Ops
-	,	rps10closeintf		/*close */
+	,	NULL		/*close */
 	,	&OurInterface
 	,	(void*)&OurImports
 	,	&interfprivate); 
 }
 
-#define	DEVICE	"WTI RPS10 Power Switch"
-
-#define N_(text)	(text)
-#define _(text)		dgettext(ST_TEXTDOMAIN, text)
-
 /*
  *	This was written for a Western Telematic Inc. (WTI) 
  *      Remote Power Switch - RPS-10M. 
@@ -203,18 +156,13 @@ struct cntrlr_str {
   char * node;          /* name of the node attached to this outlet */
 };
 
-struct WTI_RPS10 {
-  const char *	WTIid;
-
-  char *	idinfo;  /* ??? What's this for Alan ??? */
-  char *	unitid;  /* ??? What's this for Alan ??? */
+struct pluginDevice {
+  StonithPlugin	sp;
+  const char *	pluginid;
+  const char *	idinfo;
 
   int		fd;      /* FD open to the serial port */
 
-  int		config;  /* 0 if not configured, 
-                            1 if configured with rps10_set_config_info() 
-                                   or rps10_set_config_file()
-                          */
   char *	device;  /* Serial device name to use to communicate 
                             to this RPS10
 			  */
@@ -230,39 +178,35 @@ struct WTI_RPS10 {
 };
 
 /* This string is used to identify this type of object in the config file */
-static const char * WTIid = "WTI_RPS10";
+static const char * pluginid = "WTI_RPS10";
 static const char * NOTwtiid = "OBJECT DESTROYED: (WTI RPS-10)";
 
+#include "stonith_config_xml.h"
+
+#define XML_RPS10_SHORTDESC \
+	XML_PARM_SHORTDESC_BEGIN("en") \
+	"Value in the format \"serial_device remotenode outlet [remotenode outlet]...\"" \
+	XML_PARM_SHORTDESC_END
+
+#define XML_RPS10_LONGDESC \
+	XML_PARM_LONGDESC_BEGIN("en") \
+	"The RPS-10 STONITH device configuration information in the format \"serial_device remotenode outlet [remotenode outlet]...\"" \
+	XML_PARM_LONGDESC_END
+
+#define XML_RPS10_PARM \
+	XML_PARAMETER_BEGIN(ST_RPS10, "string", "1") \
+	  XML_RPS10_SHORTDESC \
+	  XML_RPS10_LONGDESC \
+	XML_PARAMETER_END
+
+static const char *rps10XML = 
+  XML_PARAMETERS_BEGIN
+    XML_RPS10_PARM
+  XML_PARAMETERS_END;
+
 /* WTIpassword - The fixed string ^B^X^X^B^X^X */
 static const char WTIpassword[7] = {2,24,24,2,24,24,0}; 
 
-#ifndef DEBUG
-#define DEBUG 0
-#endif
-static int gbl_debug = DEBUG;
-
-#define	ISWTIRPS10(i)	(((i)!= NULL && (i)->pinfo != NULL)	\
-	&& ((struct WTI_RPS10 *)(i->pinfo))->WTIid == WTIid)
-
-#define	ISCONFIGED(i)	(ISWTIRPS10(i) && ((struct WTI_RPS10 *)(i->pinfo))->config)
-
-#ifndef MALLOCT
-#	define     MALLOCT(t)      ((t *)(MALLOC(sizeof(t)))) 
-#endif
-
-#define WHITESPACE	" \t\n\r\f"
-
-#define	REPLSTR(s,v)	{					\
-			if ((s) != NULL) {			\
-				FREE(s);			\
-				(s)=NULL;			\
-			}					\
-			(s) = STRDUP(v);			\
-			if ((s) == NULL) {			\
-				syslog(LOG_ERR, _("out of memory"));\
-			}					\
-			}
-
 /*
  *	Different expect strings that we get from the WTI_RPS10
  *	Remote Power Controllers...
@@ -294,84 +238,43 @@ static struct Etoken WTItokOn[] =	{ {"On", 0, 0}, {NULL,0,0}};
 /* Accept either a CR/NL or an NL/CR */
 static struct Etoken WTItokCRNL[] =	{ {"\n\r",0,0},{"\r\n",0,0},{NULL,0,0}};
 
-static int	RPSConnect(struct WTI_RPS10 * ctx);
-static int	RPSDisconnect(struct WTI_RPS10 * ctx);
+static int	RPSConnect(struct pluginDevice * ctx);
+static int	RPSDisconnect(struct pluginDevice * ctx);
 
-static int	RPSReset(struct WTI_RPS10*, char unit_id, const char * rebootid);
+static int	RPSReset(struct pluginDevice*, char unit_id, const char * rebootid);
 #if defined(ST_POWERON) 
-static int	RPSOn(struct WTI_RPS10*, char unit_id, const char * rebootid);
+static int	RPSOn(struct pluginDevice*, char unit_id, const char * rebootid);
 #endif
 #if defined(ST_POWEROFF) 
-static int	RPSOff(struct WTI_RPS10*, char unit_id, const char * rebootid);
+static int	RPSOff(struct pluginDevice*, char unit_id, const char * rebootid);
 #endif
-static signed char RPSNametoOutlet ( struct WTI_RPS10 * ctx, const char * host );
+static signed char RPSNametoOutlet ( struct pluginDevice * ctx, const char * host );
 
-static int RPS_parse_config_info(struct WTI_RPS10* ctx, const char * info);
+static int RPS_parse_config_info(struct pluginDevice* ctx, const char * info);
 
-
-/*
- *	We do these things a lot.  Here are a few shorthand macros.
- */
-
-#define	SEND(outlet, cmd, timeout)		{                       \
-	int return_val = RPSSendCommand(ctx, outlet, cmd, timeout);     \
-	if (return_val != S_OK)  return return_val;                     \
-}
-
-#define	EXPECT(p,t)	{						\
-			if (RPSLookFor(ctx, p, t) < 0)			\
-				return(errno == ETIMEDOUT		\
-			?	S_TIMEOUT : S_OOPS);			\
-			}
-#ifdef WEDONTUSETHESE
-
-#define	NULLEXPECT(p,t)	{						\
-				if (RPSLookFor(ctx, p, t) < 0)		\
-					return(NULL);			\
-			}
-
-#define	SNARF(s, to)	{						\
-				if (RPSScanLine(ctx,to,(s),sizeof(s))	\
-				!=	S_OK)				\
-					return(S_OOPS);			\
-			}
-
-#define	NULLSNARF(s, to)	{					\
-				if (RPSScanLine(ctx,to,(s),sizeof(s))	\
-				!=	S_OK)				\
-					return(NULL);			\
-				}
-
-#endif
-
-/* Look for any of the given patterns.  We don't care which */
-
-static int
-RPSLookFor(struct WTI_RPS10* ctx, struct Etoken * tlist, int timeout)
-{
-	int	rc;
-	if ((rc = EXPECT_TOK(ctx->fd, tlist, timeout, NULL, 0)) < 0) {
-		syslog(LOG_ERR, _("Did not find string: '%s' from " DEVICE ".")
-		,	tlist[0].string);
-		RPSDisconnect(ctx);
-		return(-1);
+#define        SENDCMD(outlet, cmd, timeout)              { 		\
+		int ret_val = RPSSendCommand(ctx, outlet, cmd, timeout);\
+		if (ret_val != S_OK) {					\
+			return ret_val;					\
+		}							\
 	}
-	return(rc);
-}
-
 
 /*
  * RPSSendCommand - send a command to the specified outlet
  */
 static int
-RPSSendCommand (struct WTI_RPS10 *ctx, char outlet, char command, int timeout)
+RPSSendCommand (struct pluginDevice *ctx, char outlet, char command, int timeout)
 {
 	char            writebuf[10]; /* all commands are 9 chars long! */
 	int		return_val;  /* system call result */
 	fd_set          rfds, wfds, xfds;
-				     /*  list of FDs for select() */
 	struct timeval 	tv;	     /*  */
 
+				     /*  list of FDs for select() */
+	if (Debug) {
+		LOG(PIL_DEBUG, "%s:called.", __FUNCTION__);
+	}
+
 	FD_ZERO(&rfds);
 	FD_ZERO(&wfds);
 	FD_ZERO(&xfds);
@@ -379,7 +282,9 @@ RPSSendCommand (struct WTI_RPS10 *ctx, char outlet, char command, int timeout)
 	snprintf (writebuf, sizeof(writebuf), "%s%c%c\r",
 		  WTIpassword, outlet, command);
 
-	if (gbl_debug) printf ("Sending %s\n", writebuf);
+	if (Debug) {
+		LOG(PIL_DEBUG, "Sending %s\n", writebuf);
+	}
 
 	/* Make sure the serial port won't block on us. use select()  */
 	FD_SET(ctx->fd, &wfds);
@@ -391,21 +296,21 @@ RPSSendCommand (struct WTI_RPS10 *ctx, char outlet, char command, int timeout)
 	return_val = select(ctx->fd+1, NULL, &wfds,&xfds, &tv);
 	if (return_val == 0) {
 		/* timeout waiting on serial port */
-		syslog (LOG_ERR, "%s: Timeout writing to %s",
-			WTIid, ctx->device);
+		LOG(PIL_CRIT, "%s: Timeout writing to %s",
+			pluginid, ctx->device);
 		return S_TIMEOUT;
 	} else if ((return_val == -1) || FD_ISSET(ctx->fd, &xfds)) {
 		/* an error occured */
-		syslog (LOG_ERR, "%s: Error before writing to %s: %s",
-			WTIid, ctx->device, strerror(errno));		
+		LOG(PIL_CRIT, "%s: Error before writing to %s: %s",
+			pluginid, ctx->device, strerror(errno));		
 		return S_OOPS;
 	}
 
 	/* send the command */
 	if (write(ctx->fd, writebuf, strlen(writebuf)) != 
 			(int)strlen(writebuf)) {
-		syslog (LOG_ERR, "%s: Error writing to  %s : %s",
-			WTIid, ctx->device, strerror(errno));
+		LOG(PIL_CRIT, "%s: Error writing to  %s : %s",
+			pluginid, ctx->device, strerror(errno));
 		return S_OOPS;
 	}
 
@@ -418,39 +323,51 @@ RPSSendCommand (struct WTI_RPS10 *ctx, char outlet, char command, int timeout)
  * RPSReset - Reset (power-cycle) the given outlet id 
  */
 static int
-RPSReset(struct WTI_RPS10* ctx, char unit_id, const char * rebootid)
+RPSReset(struct pluginDevice* ctx, char unit_id, const char * rebootid)
 {
 
-	if (gbl_debug) printf ("Calling RPSReset (%s)\n", WTIid);
-	
+	if (Debug) {
+		LOG(PIL_DEBUG, "%s:called.", __FUNCTION__);
+	}
+
 	if (ctx->fd < 0) {
-		syslog(LOG_ERR, "%s: device %s is not open!", WTIid, 
+		LOG(PIL_CRIT, "%s: device %s is not open!", pluginid, 
 		       ctx->device);
 		return S_OOPS;
 	}
 
 	/* send the "toggle power" command */
-	SEND(unit_id, 'T', 10);
+	SENDCMD(unit_id, 'T', 10);
 
 	/* Expect "Plug 0 Off" */
 	/* Note: If asked to control "*", the RPS10 will report all units it
 	 * separately; however, we don't know how many, so we can only wait
 	 * for the first unit to report something and then wait until the
 	 * "Complete" */
-	EXPECT(WTItokPlug, 5);
-	if (gbl_debug)	printf ("Got Plug\n");
-	EXPECT(WTItokOutlet, 2);
-	if (gbl_debug) printf ("Got Outlet #\n");
-	EXPECT(WTItokOff, 2);
-	if (gbl_debug) printf ("Got Off\n");	
-	EXPECT(WTItokCRNL, 2);
-	syslog(LOG_INFO, _("Host %s being rebooted."), rebootid);
+	EXPECT(ctx->fd, WTItokPlug, 5);
+	if (Debug) {
+		LOG(PIL_DEBUG, "Got Plug\n");
+	}
+	EXPECT(ctx->fd, WTItokOutlet, 2);
+	if (Debug) {
+		LOG(PIL_DEBUG, "Got Outlet #\n");
+	}
+	EXPECT(ctx->fd, WTItokOff, 2);
+	if (Debug) {
+		LOG(PIL_DEBUG, "Got Off\n");
+	}	
+	EXPECT(ctx->fd, WTItokCRNL, 2);
+	LOG(PIL_INFO, "Host is being rebooted: %s", rebootid);
 	
 	/* Expect "Complete" */
-	EXPECT(WTItokComplete, 14);
-	if (gbl_debug) printf ("Got Complete\n");
-	EXPECT(WTItokCRNL, 2);
-	if (gbl_debug) printf ("Got NL\n");
+	EXPECT(ctx->fd, WTItokComplete, 14);
+	if (Debug) {
+		LOG(PIL_DEBUG, "Got Complete\n");
+	}
+	EXPECT(ctx->fd, WTItokCRNL, 2);
+	if (Debug) {
+		LOG(PIL_DEBUG, "Got NL\n");
+	}
 	
 	return(S_OK);
 
@@ -462,28 +379,31 @@ RPSReset(struct WTI_RPS10* ctx, char unit_id, const char * rebootid)
  * RPSOn - Turn OFF the given outlet id 
  */
 static int
-RPSOn(struct WTI_RPS10* ctx, char unit_id, const char * host)
+RPSOn(struct pluginDevice* ctx, char unit_id, const char * host)
 {
+	if (Debug) {
+		LOG(PIL_DEBUG, "%s:called.", __FUNCTION__);
+	}
 
 	if (ctx->fd < 0) {
-		syslog(LOG_ERR, "%s: device %s is not open!", WTIid, 
+		LOG(PIL_CRIT, "%s: device %s is not open!", pluginid, 
 		       ctx->device);
 		return S_OOPS;
 	}
 
 	/* send the "On" command */
-	SEND(unit_id, '1', 10);
+	SENDCMD(unit_id, '1', 10);
 
 	/* Expect "Plug 0 On" */
-	EXPECT(WTItokPlug, 5);
-	EXPECT(WTItokOutlet, 2);
-	EXPECT(WTItokOn, 2);
-	EXPECT(WTItokCRNL, 2);
-	syslog(LOG_INFO, _("Host %s being turned on."), host);
+	EXPECT(ctx->fd, WTItokPlug, 5);
+	EXPECT(ctx->fd, WTItokOutlet, 2);
+	EXPECT(ctx->fd, WTItokOn, 2);
+	EXPECT(ctx->fd, WTItokCRNL, 2);
+	LOG(PIL_INFO, "Host is being turned on: %s", host);
 	
 	/* Expect "Complete" */
-	EXPECT(WTItokComplete, 5);
-	EXPECT(WTItokCRNL, 2);
+	EXPECT(ctx->fd, WTItokComplete, 5);
+	EXPECT(ctx->fd, WTItokCRNL, 2);
 
 	return(S_OK);
 
@@ -496,28 +416,32 @@ RPSOn(struct WTI_RPS10* ctx, char unit_id, const char * host)
  * RPSOff - Turn Off the given outlet id 
  */
 static int
-RPSOff(struct WTI_RPS10* ctx, char unit_id, const char * host)
+RPSOff(struct pluginDevice* ctx, char unit_id, const char * host)
 {
 
+	if (Debug) {
+		LOG(PIL_DEBUG, "%s:called.", __FUNCTION__);
+	}
+	
 	if (ctx->fd < 0) {
-		syslog(LOG_ERR, "%s: device %s is not open!", WTIid, 
+		LOG(PIL_CRIT, "%s: device %s is not open!", pluginid, 
 		       ctx->device);
 		return S_OOPS;
 	}
 
 	/* send the "Off" command */
-	SEND(unit_id, '0', 10);
+	SENDCMD(unit_id, '0', 10);
 
 	/* Expect "Plug 0 Off" */
-	EXPECT(WTItokPlug, 5);
-	EXPECT(WTItokOutlet, 2);
-	EXPECT(WTItokOff, 2);
-	EXPECT(WTItokCRNL, 2);
-	syslog(LOG_INFO, _("Host %s being turned on."), host);
+	EXPECT(ctx->fd, WTItokPlug, 5);
+	EXPECT(ctx->fd, WTItokOutlet, 2);
+	EXPECT(ctx->fd, WTItokOff, 2);
+	EXPECT(ctx->fd, WTItokCRNL, 2);
+	LOG(PIL_INFO, "Host is being turned on: %s", host);
 	
 	/* Expect "Complete" */
-	EXPECT(WTItokComplete, 5);
-	EXPECT(WTItokCRNL, 2);
+	EXPECT(ctx->fd, WTItokComplete, 5);
+	EXPECT(ctx->fd, WTItokCRNL, 2);
 
 	return(S_OK);
 
@@ -535,22 +459,17 @@ RPSOff(struct WTI_RPS10* ctx, char unit_id, const char * host)
  *    S_OK   - if the stonith device is reachable and online.
  */
 static int
-rps10_status(Stonith  *s)
+rps10_status(StonithPlugin  *s)
 {
-	struct WTI_RPS10*	ctx;
+	struct pluginDevice*	ctx;
 	
-	if (gbl_debug) printf ("Calling rps10_status (%s)\n", WTIid);
+	if (Debug) {
+		LOG(PIL_DEBUG, "%s:called.", __FUNCTION__);
+	}
 	
-	if (!ISWTIRPS10(s)) {
-		syslog(LOG_ERR, "invalid argument to RPS_status");
-		return(S_OOPS);
-	}
-	if (!ISCONFIGED(s)) {
-		syslog(LOG_ERR
-		,	"unconfigured stonith object in RPS_status");
-		return(S_OOPS);
-	}
-	ctx = (struct WTI_RPS10*) s->pinfo;
+	ERRIFNOTCONFIGED(s,S_OOPS);
+
+	ctx = (struct pluginDevice*) s;
 	if (RPSConnect(ctx) != S_OK) {
 		return(S_OOPS);
 	}
@@ -576,35 +495,29 @@ rps10_status(Stonith  *s)
  *         of null-terminated malloc'ed strings.
  */
 static char **
-rps10_hostlist(Stonith  *s)
+rps10_hostlist(StonithPlugin  *s)
 {
 	char **		ret = NULL;	/* list to return */
 	int 		i;
 	int 		j;
-	struct WTI_RPS10*	ctx;
+	struct pluginDevice*	ctx;
 
-	if (gbl_debug) printf ("Calling rps10_hostlist (%s)\n", WTIid);
-	
-	if (!ISWTIRPS10(s)) {
-		syslog(LOG_ERR, "invalid argument to RPS_list_hosts");
-		return(NULL);
-	}
-	if (!ISCONFIGED(s)) {
-		syslog(LOG_ERR
-		,	"unconfigured stonith object in RPS_list_hosts");
-		return(NULL);
+	if (Debug) {
+		LOG(PIL_DEBUG, "%s:called.", __FUNCTION__);
 	}
 
-	ctx = (struct WTI_RPS10*) s->pinfo;
+	ERRIFNOTCONFIGED(s,NULL);
+
+	ctx = (struct pluginDevice*) s;
 
 	if (ctx->unit_count >= 1) {
 		ret = (char **)MALLOC((ctx->unit_count+1)*sizeof(char*));
 		if (ret == NULL) {
-			syslog(LOG_ERR, "out of memory");
-			return NULL;
+			LOG(PIL_CRIT, "out of memory");
+			return ret;
 		}
 		ret[ctx->unit_count]=NULL; /* null terminate the array */
-		for (i=0; i<ctx->unit_count; i++) {
+		for (i=0; i < ctx->unit_count; i++) {
 			ret[i] = STRDUP(ctx->controllers[i].node);
 			if (ret[i] == NULL) {
 				for(j=0; j<i; j++) {
@@ -615,29 +528,9 @@ rps10_hostlist(Stonith  *s)
 			}
 		} /* end for each possible outlet */
 	} /* end if any outlets are configured */
-
 	return(ret);
 } /* end si_hostlist() */
 
-/*
- * rps10_free_hostlist - free the result from rps10_hostlist() 
- */
-static void
-rps10_free_hostlist (char ** hlist)
-{
-	char **	hl = hlist;
-	if (hl == NULL) {
-		return;
-	}
-	while (*hl) {
-		FREE(*hl);
-		*hl = NULL;
-		++hl;
-	}
-	FREE(hlist);
-}
-
-
 /*
  *	Parse the given configuration information, and stash
  *      it away...
@@ -675,16 +568,14 @@ rps10_free_hostlist (char ** hlist)
  */
 
 static int
-RPS_parse_config_info(struct WTI_RPS10* ctx, const char * info)
+RPS_parse_config_info(struct pluginDevice* ctx, const char * info)
 {
 	char *copy;
 	char *token;
 	char *outlet, *node;
 
-
-	if (ctx->config) {
-		/* The module is already configured. */
-		return(S_OOPS);
+	if (Debug) {
+		LOG(PIL_DEBUG, "%s:called.", __FUNCTION__);
 	}
 
 	/* strtok() is nice to use to parse a string with 
@@ -695,7 +586,7 @@ RPS_parse_config_info(struct WTI_RPS10* ctx, const char * info)
 
 	copy = STRDUP(info);
 	if (!copy) {
-		syslog(LOG_ERR, "out of memory");
+		LOG(PIL_CRIT, "out of memory");
 		return S_OOPS;
 	}
 
@@ -703,14 +594,14 @@ RPS_parse_config_info(struct WTI_RPS10* ctx, const char * info)
 	token = strtok (copy, " \t");
 
 	if (!token) {
-		syslog(LOG_ERR, "%s: Can't find serial device on config line '%s'",
-		       WTIid, info);
+		LOG(PIL_CRIT, "%s: Can't find serial device on config line '%s'",
+		       pluginid, info);
 		goto token_error;		
 	}
 
 	ctx->device = STRDUP(token);
 	if (!ctx->device) {
-		syslog(LOG_ERR, "out of memory");
+		LOG(PIL_CRIT, "out of memory");
 		goto token_error;
 	}
 
@@ -725,10 +616,10 @@ RPS_parse_config_info(struct WTI_RPS10* ctx, const char * info)
 		    || !( ((outlet_id >= '0') && (outlet_id <= '9'))
 			|| (outlet_id == '*') || (outlet_id == 'A') )
 		   ) {
-			syslog(LOG_ERR
+			LOG(PIL_CRIT
 			, "%s: the outlet_id %s must be between"
 			" 0 and 9 or '*' / 'A'",
-			       WTIid, outlet);
+			       pluginid, outlet);
 			goto token_error;
 		}
 		
@@ -740,9 +631,9 @@ RPS_parse_config_info(struct WTI_RPS10* ctx, const char * info)
 		}
 		
 		if (ctx->unit_count >= WTI_NUM_CONTROLLERS) {
-			syslog(LOG_ERR, 
+			LOG(PIL_CRIT, 
 				"%s: Tried to configure too many controllers",
-				WTIid);
+				pluginid);
 			goto token_error;
 		}
 		
@@ -757,11 +648,14 @@ RPS_parse_config_info(struct WTI_RPS10* ctx, const char * info)
 	 * parsing with strtok()
 	 */
 	FREE(copy);
-	ctx->config = 1;
 	return ((ctx->unit_count > 0) ? S_OK : S_BADCONFIG);
 
 token_error:
 	FREE(copy);
+	if (ctx->device) {
+		FREE(ctx->device);
+		ctx->device = NULL;
+	}
 	return(S_BADCONFIG);
 }
 
@@ -773,22 +667,26 @@ token_error:
  *
  */
 static void dtrtoggle(int fd) {
-    struct termios tty, old;
-    int sec = 2;
+	struct termios tty, old;
+	int sec = 2;
     
-    if (gbl_debug) printf ("Calling dtrtoggle (%s)\n", WTIid);
+	if (Debug) {
+		LOG(PIL_DEBUG, "%s:called.", __FUNCTION__);
+	}
+	
+	tcgetattr(fd, &tty);
+	tcgetattr(fd, &old);
+	cfsetospeed(&tty, B0);
+	cfsetispeed(&tty, B0);
+	tcsetattr(fd, TCSANOW, &tty);
+	if (sec>0) {
+		sleep(sec);
+		tcsetattr(fd, TCSANOW, &old);
+	}
     
-    tcgetattr(fd, &tty);
-    tcgetattr(fd, &old);
-    cfsetospeed(&tty, B0);
-    cfsetispeed(&tty, B0);
-    tcsetattr(fd, TCSANOW, &tty);
-    if (sec>0) {
-      sleep(sec);
-      tcsetattr(fd, TCSANOW, &old);
-    }
-    
-    if (gbl_debug) printf ("dtrtoggle Complete (%s)\n", WTIid);
+	if (Debug) {
+		LOG(PIL_DEBUG, "dtrtoggle Complete (%s)\n", pluginid);
+	}
 }
 
 /*
@@ -807,17 +705,25 @@ static void dtrtoggle(int fd) {
  *
  */
 static int
-RPSConnect(struct WTI_RPS10 * ctx)
+RPSConnect(struct pluginDevice * ctx)
 {
+	if (Debug) {
+		LOG(PIL_DEBUG, "%s:called.", __FUNCTION__);
+	}
   	  
 	/* Open the serial port if it isn't already open */
 	if (ctx->fd < 0) {
 		struct termios tio;
 
+		if (OurImports->TtyLock(ctx->device) < 0) {
+			LOG(PIL_CRIT, "%s: TtyLock failed.", pluginid);
+			return S_OOPS;
+		}
+
 		ctx->fd = open (ctx->device, O_RDWR);
 		if (ctx->fd <0) {
-			syslog (LOG_ERR, "%s: Can't open %s : %s",
-				WTIid, ctx->device, strerror(errno));
+			LOG(PIL_CRIT, "%s: Can't open %s : %s",
+				pluginid, ctx->device, strerror(errno));
 			return S_OOPS;
 		}
 
@@ -836,17 +742,19 @@ RPSConnect(struct WTI_RPS10 * ctx)
 		tio.c_lflag = ICANON;
 
 		if (tcsetattr (ctx->fd, TCSANOW, &tio) < 0) {
-			syslog (LOG_ERR, "%s: Can't set attributes %s : %s",
-				WTIid, ctx->device, strerror(errno));
+			LOG(PIL_CRIT, "%s: Can't set attributes %s : %s",
+				pluginid, ctx->device, strerror(errno));
 			close (ctx->fd);
+			OurImports->TtyUnlock(ctx->device);
 			ctx->fd=-1;
 			return S_OOPS;
 		}
 		/* flush all data to and fro the serial port before we start */
 		if (tcflush (ctx->fd, TCIOFLUSH) < 0) {
-			syslog (LOG_ERR, "%s: Can't flush %s : %s",
-				WTIid, ctx->device, strerror(errno));
+			LOG(PIL_CRIT, "%s: Can't flush %s : %s",
+				pluginid, ctx->device, strerror(errno));
 			close (ctx->fd);
+			OurImports->TtyUnlock(ctx->device);
 			ctx->fd=-1;
 			return S_OOPS;		
 		}
@@ -864,29 +772,42 @@ RPSConnect(struct WTI_RPS10 * ctx)
 	   a broken serial cable, which doesn't connect hardware
 	   flow control.
 	*/
-	if (gbl_debug) printf ("Waiting for READY\n");
-	EXPECT(WTItokReady, 12);
-	if (gbl_debug) printf ("Got READY\n");
-	EXPECT(WTItokCRNL, 2);
-	if (gbl_debug) printf ("Got NL\n");
+	if (Debug) {
+		LOG(PIL_DEBUG, "Waiting for READY\n");
+	}
+	EXPECT(ctx->fd, WTItokReady, 12);
+	if (Debug) {
+		LOG(PIL_DEBUG, "Got READY\n");
+	}
+	EXPECT(ctx->fd, WTItokCRNL, 2);
+	if (Debug) {
+		LOG(PIL_DEBUG, "Got NL\n");
+	}
 
   return(S_OK);
 }
 
 static int
-RPSDisconnect(struct WTI_RPS10 * ctx)
+RPSDisconnect(struct pluginDevice * ctx)
 {
 
-  if (ctx->fd >= 0) {
-    /* Flush the serial port, we don't care what happens to the characters
-       and failing to do this can cause close to hang.
-    */
-    tcflush(ctx->fd, TCIOFLUSH);
-    close (ctx->fd);
-  }
-  ctx->fd = -1;
+	if (Debug) {
+		LOG(PIL_DEBUG, "%s:called.", __FUNCTION__);
+	}
 
-  return S_OK;
+	if (ctx->fd >= 0) {
+		/* Flush the serial port, we don't care what happens to the 
+		 * characters and failing to do this can cause close to hang.
+		 */
+		tcflush(ctx->fd, TCIOFLUSH);
+		close (ctx->fd);
+		if (ctx->device != NULL) {
+			OurImports->TtyUnlock(ctx->device);
+		}
+	}
+	ctx->fd = -1;
+
+	return S_OK;
 } 
 
 /*
@@ -898,28 +819,24 @@ RPSDisconnect(struct WTI_RPS10 * ctx)
  * 
  */
 static signed char
-RPSNametoOutlet ( struct WTI_RPS10 * ctx, const char * host )
+RPSNametoOutlet ( struct pluginDevice * ctx, const char * host )
 {
 	int i=0;
-	char *shost;
 
-	if ( (shost = STRDUP(host)) == NULL) {
-		syslog(LOG_ERR, "strdup failed in RPSNametoOutlet");
-		return -1;
+	if (Debug) {
+		LOG(PIL_DEBUG, "%s:called.", __FUNCTION__);
 	}
-	g_strdown(shost);
-		
+
 	/* scan the controllers[] array to see if this host is there */
 	for (i=0;i<ctx->unit_count;i++) {
 		/* return the outlet id */
 		if ( ctx->controllers[i].node 
-		    && !strcmp(host, ctx->controllers[i].node)) {
+		    && !strcasecmp(host, ctx->controllers[i].node)) {
 			/* found it! */
 			break;
 		}
 	}
 	
-	free(shost);
 	if (i == ctx->unit_count) {
 		return -1;
 	} else {
@@ -935,25 +852,20 @@ RPSNametoOutlet ( struct WTI_RPS10 * ctx, const char * host )
  *          on the stonith device.
  */
 static int
-rps10_reset_req(Stonith * s, int request, const char * host)
+rps10_reset_req(StonithPlugin * s, int request, const char * host)
 {
 	int	rc = S_OK;
 	int	lorc = S_OK;
 	signed char outlet_id = -1;
-	struct WTI_RPS10*	ctx;
+	struct pluginDevice*	ctx;
 	
-	if (gbl_debug) printf ("Calling rps10_reset (%s)\n", WTIid);
-	
-	if (!ISWTIRPS10(s)) {
-		syslog(LOG_ERR, "invalid argument to RPS_reset_host");
-		return(S_OOPS);
+	if (Debug) {
+		LOG(PIL_DEBUG, "%s:called.", __FUNCTION__);
 	}
-	if (!ISCONFIGED(s)) {
-		syslog(LOG_ERR
-		,	"unconfigured stonith object in RPS_reset_host");
-		return(S_OOPS);
-	}
-	ctx = (struct WTI_RPS10*) s->pinfo;
+
+	ERRIFNOTCONFIGED(s,S_OOPS);
+
+	ctx = (struct pluginDevice*) s;
 
 	if ((rc = RPSConnect(ctx)) != S_OK) {
 		return(rc);
@@ -962,9 +874,8 @@ rps10_reset_req(Stonith * s, int request, const char * host)
 	outlet_id = RPSNametoOutlet(ctx, host);
 
 	if (outlet_id < 0) {
-		syslog(LOG_WARNING, _("%s %s "
-				      "doesn't control host [%s]."), 
-		       ctx->idinfo, ctx->unitid, host);
+		LOG(PIL_WARN, "%s: %s doesn't control host [%s]"
+		,	pluginid, ctx->device, host );
 		RPSDisconnect(ctx);
 		return(S_BADHOST);
 	}
@@ -995,109 +906,95 @@ rps10_reset_req(Stonith * s, int request, const char * host)
 }
 
 /*
- *	Parse the information in the given configuration file,
+ *	Parse the information in the given string,
  *	and stash it away...
  */
 static int
-rps10_set_config_file(Stonith* s, const char * configname)
+rps10_set_config(StonithPlugin* s, StonithNVpair* list)
 {
-	FILE *	cfgfile;
+	struct pluginDevice*	ctx;
+	StonithNamesToGet	namestocopy [] =
+	{	{ST_RPS10,	NULL}
+	,	{NULL,		NULL}
+	};
+	int rc=0;
 
-	char	RPSid[256];
+	if (Debug) {
+		LOG(PIL_DEBUG, "%s:called.", __FUNCTION__);
+	}
 
-	struct WTI_RPS10*	ctx;
+	ERRIFWRONGDEV(s,S_OOPS);
 
-	if (!ISWTIRPS10(s)) {
-		syslog(LOG_ERR, "invalid argument to RPS_set_configfile");
+	if (s->isconfigured) {
+		/* The module is already configured. */
 		return(S_OOPS);
 	}
-	ctx = (struct WTI_RPS10*) s->pinfo;
 
-	if ((cfgfile = fopen(configname, "r")) == NULL)  {
-		syslog(LOG_ERR, _("Cannot open %s"), configname);
-		return(S_BADCONFIG);
+	ctx = (struct pluginDevice*) s;
+
+	if((rc = OurImports->CopyAllValues(namestocopy, list)) != S_OK){
+		LOG(PIL_DEBUG , "get all calues failed");	
+		return rc;
 	}
-
-	while (fgets(RPSid, sizeof(RPSid), cfgfile) != NULL){
-
-		switch (RPSid[0]){
-			case '\0': case '\n': case '\r': case '#':
-			continue;
-		}
-
-		/* We can really only handle one line. Wimpy. */
-		return RPS_parse_config_info(ctx, RPSid);
-	}
-	return(S_BADCONFIG);
+	
+	rc = RPS_parse_config_info(ctx, namestocopy[0].s_value);	
+	FREE(namestocopy[0].s_value);
+	return rc;
 }
 
 /*
- *	rps10_set_config_info - API entry point to process one line of config info 
- *       for this particular device.
- *
- *      Parse the config information in the given string, and stash it away...
+ *  Return the Stonith plugin configuration parameter 
  *
  */
-static int
-rps10_set_config_info(Stonith* s, const char * info)
+static const char**
+rps10_get_confignames(StonithPlugin* p)
 {
-	struct WTI_RPS10* ctx;
+	static const char *	Rps10Params[] = {ST_RPS10 ,NULL };
 
-	if (!ISWTIRPS10(s)) {
-		syslog(LOG_ERR, "RPS_provide_config_info: invalid argument");
-		return(S_OOPS);
+	if (Debug) {
+		LOG(PIL_DEBUG, "%s:called.", __FUNCTION__);
 	}
-	ctx = (struct WTI_RPS10 *)s->pinfo;
 
-	return(RPS_parse_config_info(ctx, info));
+	return Rps10Params;
 }
 
 /*
  * rps10_getinfo - API entry point to retrieve something from the handle
  */
 static const char *
-rps10_getinfo(Stonith * s, int reqtype)
+rps10_getinfo(StonithPlugin * s, int reqtype)
 {
-	struct WTI_RPS10* ctx;
+	struct pluginDevice* ctx;
 	const char *	ret;
 
-	if (!ISWTIRPS10(s)) {
-		syslog(LOG_ERR, "RPS_idinfo: invalid argument");
-		return NULL;
+	if (Debug) {
+		LOG(PIL_DEBUG, "%s:called.", __FUNCTION__);
 	}
+
+	ERRIFWRONGDEV(s,NULL);
+
 	/*
 	 *	We look in the ST_TEXTDOMAIN catalog for our messages
 	 */
-	ctx = (struct WTI_RPS10 *)s->pinfo;
+	ctx = (struct pluginDevice *)s;
 
 	switch (reqtype) {
 		case ST_DEVICEID:
 			ret = ctx->idinfo;
 			break;
-
-		case ST_CONF_INFO_SYNTAX:
-			ret = _("<serial_device> <node> <outlet> "
-			"[ <node> <outlet> [...] ]\n"
-			"All tokens are white-space delimited.\n");
+		case ST_DEVICENAME:
+			ret = ctx->device;
 			break;
-
-		case ST_CONF_FILE_SYNTAX:
-			ret = _("<serial_device> <node> <outlet> "
-			"[ <node> <outlet> [...] ]\n"
-			"All tokens are white-space delimited.\n"
-			"Blank lines and lines beginning with # are ignored");
-			break;
-
 		case ST_DEVICEDESCR:
-			ret = _("Western Telematic Inc. (WTI) "
-			"Remote Power Switch - RPS-10M.\n");
+			ret = "Western Telematic Inc. (WTI) "
+			"Remote Power Switch - RPS-10M.\n";
 			break;
-
-
 		case ST_DEVICEURL:
 			ret = "http://www.wti.com/";
 			break;
-
+		case ST_CONF_XML:		/* XML metadata */
+			ret = rps10XML;
+			break;
 		default:
 			ret = NULL;
 			break;
@@ -1109,17 +1006,20 @@ rps10_getinfo(Stonith * s, int reqtype)
  * rps10_destroy - API entry point to destroy a WTI_RPS10 Stonith object.
  */
 static void
-rps10_destroy(Stonith *s)
+rps10_destroy(StonithPlugin *s)
 {
-	struct WTI_RPS10* ctx;
+	struct pluginDevice* ctx;
+	int i;
 
-	if (!ISWTIRPS10(s)) {
-		syslog(LOG_ERR, "wti_rps10_del: invalid argument");
-		return;
+	if (Debug) {
+		LOG(PIL_DEBUG, "%s:called.", __FUNCTION__);
 	}
-	ctx = (struct WTI_RPS10 *)s->pinfo;
 
-	ctx->WTIid = NOTwtiid;
+	VOIDERRIFWRONGDEV(s);
+
+	ctx = (struct pluginDevice *)s;
+
+	ctx->pluginid = NOTwtiid;
 
 	/*  close the fd if open and set ctx->fd to invalid */
 	RPSDisconnect(ctx);
@@ -1128,39 +1028,41 @@ rps10_destroy(Stonith *s)
 		FREE(ctx->device);
 		ctx->device = NULL;
 	}
-	if (ctx->idinfo != NULL) {
-		FREE(ctx->idinfo);
-		ctx->idinfo = NULL;
-	}
-	if (ctx->unitid != NULL) {
-		FREE(ctx->unitid);
-		ctx->unitid = NULL;
+	if (ctx->unit_count > 0) {
+		for (i = 0; i < ctx->unit_count; i++) {
+			if (ctx->controllers[i].node != NULL) {
+				FREE(ctx->controllers[i].node);
+				ctx->controllers[i].node = NULL;
+			}
+		}
 	}
+	FREE(ctx);
 }
 
 /* 
  * rps10_new - API entry point called to create a new WTI_RPS10 Stonith device
  *          object. 
  */
-static void *
-rps10_new(void)
+static StonithPlugin * 
+rps10_new(const char *subplugin)
 {
-	struct WTI_RPS10*	ctx = MALLOCT(struct WTI_RPS10);
+	struct pluginDevice*	ctx = ST_MALLOCT(struct pluginDevice);
+
+	if (Debug) {
+		LOG(PIL_DEBUG, "%s:called.", __FUNCTION__);
+	}
 
 	if (ctx == NULL) {
-		syslog(LOG_ERR, "out of memory");
+		LOG(PIL_CRIT, "out of memory");
 		return(NULL);
 	}
 	memset(ctx, 0, sizeof(*ctx));
-	ctx->WTIid = WTIid;
+	ctx->pluginid = pluginid;
 	ctx->fd = -1;
-	ctx->config = 0;
 	ctx->unit_count = 0;
 	ctx->device = NULL;
-	ctx->idinfo = NULL;
-	ctx->unitid = NULL;
-	REPLSTR(ctx->idinfo, DEVICE);
-	REPLSTR(ctx->unitid, "unknown");
+	ctx->idinfo = DEVICE;
+	ctx->sp.s_ops = &rps10Ops;
 
-	return((void *)ctx);
+	return &(ctx->sp);
 }
