@@ -1,4 +1,3 @@
-/* $Id: apphbd.c,v 1.51.2.10 2005/08/19 15:36:22 sunjd Exp $ */
 /*
  * apphbd:	application heartbeat daemon
  *
@@ -13,19 +12,19 @@
  *
  *********************************************************************
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- * 
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /*
@@ -58,7 +57,7 @@
  *
  */
 
-#include <portability.h>
+#include <lha_internal.h>
 #include <stdio.h>
 #include <syslog.h>
 #include <sys/types.h>
@@ -91,27 +90,38 @@
 #include <clplumbing/cl_signal.h>
 #include <clplumbing/lsb_exitcodes.h>
 #include <clplumbing/coredumps.h>
-
+#include <clplumbing/cl_malloc.h>
+#include <clplumbing/cl_pidfile.h>
 
 #ifndef PIDFILE
-#	define		PIDFILE    HA_VARRUNDIR"/apphbd.pid"
+#	define		PIDFILE HA_VARRUNDIR "/apphbd.pid"
 #endif
 
+/* Start: Mirrored from ipcsocket.c */
+#ifdef SO_PEERCRED
+#	define	USE_SO_PEERCRED
+#elif HAVE_GETPEEREID
+#	define USE_GETPEEREID
+#elif defined(SCM_CREDS)
+#	define	USE_SCM_CREDS
+#else
+#	define	USE_DUMMY_CREDS
+/* This will make it compile, but attempts to authenticate
+ * will fail.  This is a stopgap measure ;-)
+ */
+#endif
+/* End: Mirrored from ipcsocket.c */
+
 const char *	cmdname = "apphbd";
 #define		DBGMIN		1
 #define		DBGDETAIL	2
 static int	usenormalpoll = TRUE;
 static int	watchdogfd = -1;
-static int	cmdline_debug_opt = 0;
 
-#ifndef SYSCONFDIR
-#	define	SYSCONFDIR "/etc"
-#endif
-#define CONFIG_FILE SYSCONFDIR "/apphbd.cf"
-#define MAXLINE 128
+#define CONFIG_FILE "./apphbd.cf"
 #define EOS '\0'
 
-#define DEFAULT_DEBUG_LEVEL	"0"
+#define DEFAULT_DEBUG_LEVEL	"3"
 #define DEFAULT_WDT_DEV		NULL
 #define DEFAULT_WDT_INTERVAL_MS	"1000"
 #define DEFAULT_REALTIME	"yes"
@@ -125,6 +135,8 @@ typedef struct apphb_client apphb_client_t;
 struct apphb_client {
 	char *			appname;	/* application name */
 	char *			appinst;	/* application name */
+	char *			curdir;		/* application starting 
+						   directory */
 	pid_t			pid;		/* application pid */
 	uid_t			uid;		/* application UID */
 	gid_t			gid;		/* application GID */
@@ -146,7 +158,6 @@ AppHBNotifyOps*	NotificationPlugins[MAXNOTIFYPLUGIN];
 int		n_Notification_Plugins = 0;
 
 static void apphb_notify(apphb_client_t* client, apphb_event_t event);
-static long get_running_pid(gboolean * anypidfile);
 static void make_daemon(void);
 static int init_start(void);
 static int init_stop(void);
@@ -267,7 +278,9 @@ apphb_client_new(struct IPC_CHANNEL* ch)
 	apphb_client_t*	ret;
 
 	ret = g_new(apphb_client_t, 1);
-
+	
+	memset(ret, 0, sizeof(*ret));
+	
 	ret->appname = NULL;
 	ret->appinst = NULL;
 	ret->ch = ch;
@@ -280,6 +293,8 @@ apphb_client_new(struct IPC_CHANNEL* ch)
 	 * NOTE: this disallows multiple outstanding calls from a client
 	 * (IMHO this is not a problem)
 	 */
+
+	ret->rcmsg.msg_buf = NULL;
 	ret->rcmsg.msg_body = &ret->rc;
 	ret->rcmsg.msg_len = sizeof(ret->rc);
 	ret->rcmsg.msg_done = NULL;
@@ -334,11 +349,15 @@ apphb_client_register(apphb_client_t* client, void* Msg,  size_t length)
 		return EINVAL;
 	}
 
+#ifndef USE_SO_PEERCRED
+	if(client->ch->farside_pid == -1) {
+		client->ch->farside_pid = msg->pid;
+	}
+#endif
 	if (msg->pid < 2 || (CL_KILL(msg->pid, 0) < 0 && errno != EPERM)
 	||	(client->ch->farside_pid != msg->pid)) {
 		return EINVAL;
 	}
-
 	client->pid = msg->pid;
 
 	/* Make sure client is who they claim to be... */
@@ -352,13 +371,14 @@ apphb_client_register(apphb_client_t* client, void* Msg,  size_t length)
 	ipc_destroy_auth(clientauth);
 	client->appname = g_strdup(msg->appname);
 	client->appinst = g_strdup(msg->appinstance);
+	client->curdir = g_strdup(msg->curdir);
 	client->uid = msg->uid;
 	client->gid = msg->gid;
 
 	if (apphbd_config.debug_level >= DBGMIN) {
 		cl_log(LOG_DEBUG
 		,	"apphb_client_register: client: [%s]/[%s] pid %ld"
-		" (uid,gid) = (%ld,%ld)"
+		" (uid,gid) = (%ld,%ld)\n"
 		,	client->appname
 		,	client->appinst
 		,	(long)client->pid
@@ -369,7 +389,7 @@ apphb_client_register(apphb_client_t* client, void* Msg,  size_t length)
 	/* Tell the plugins something happened */
 	for (j=0; j < n_Notification_Plugins; ++j) {
 		NotificationPlugins[j]->cregister(client->pid
-		,	client->appname, client->appinst
+		,	client->appname, client->appinst, client->curdir
 		,	client->uid, client->gid, client);
 	}
 	return 0;
@@ -381,6 +401,8 @@ static void
 apphb_client_remove(gpointer Client)
 {
 	apphb_client_t* client = Client;
+	cl_log(LOG_INFO, "apphb_client_remove: client: %ld"
+	,	(long)client->pid);
 	if (apphbd_config.debug_level >= DBGMIN) {
 		cl_log(LOG_DEBUG, "apphb_client_remove: client pid: %ld"
 		,	(long)client->pid);
@@ -392,6 +414,7 @@ apphb_client_remove(gpointer Client)
 	G_main_del_IPC_Channel(client->source);
 	g_free(client->appname);
 	g_free(client->appinst);
+	g_free(client->curdir);
 	memset(client, 0, sizeof(*client));
 }
 
@@ -639,7 +662,7 @@ apphb_notify(apphb_client_t* client, apphb_event_t event)
 	/* Tell the plugins something happened */
 	for (j=0; j < n_Notification_Plugins; ++j) {
 		NotificationPlugins[j]->status(client->appname
-		,	client->appinst, client->pid
+		,	client->appinst, client->curdir, client->pid
 		,	client->uid, client->gid, event);
 	}
 }
@@ -876,7 +899,9 @@ main(int argc, char ** argv)
 	int		argerr = 0;
 	const char*	cfgfile = CONFIG_FILE;
 
+	cl_malloc_forced_for_glib(); /* Must be first */
 	cl_cdtocoredir();
+	cl_enable_coredumps(TRUE);
 	cl_log_set_entity(cmdname);
 	cl_log_enable_stderr(TRUE);
 	cl_log_set_facility(LOG_USER);
@@ -899,7 +924,7 @@ main(int argc, char ** argv)
 				cfgfile = optarg;
 				break;
 			case 'd':
-				++cmdline_debug_opt;
+				++apphbd_config.debug_level;
 				break;
 			default:
 				++argerr;
@@ -928,14 +953,6 @@ main(int argc, char ** argv)
 	}
 
 	init_config(cfgfile);
-	/* -d level on command line overrides that in config file */
-	if (cmdline_debug_opt) {
-		apphbd_config.debug_level = cmdline_debug_opt;
-	}
-	if (apphbd_config.debug_level) {
-		cl_log(LOG_DEBUG, "Debug level is %d"
-		,	apphbd_config.debug_level);
-	}
 	return init_start();
 
 }
@@ -977,12 +994,12 @@ init_start()
 	struct IPC_WAIT_CONNECTION*	wconn;
 	GHashTable*	wconnattrs;
 
-	if ((pid = get_running_pid(NULL)) > 0) {
-		cl_log(LOG_CRIT, "already running: [pid %ld]."
-		,	pid);
+	if ((pid = cl_read_pidfile(PIDFILE)) > 0) {
+		cl_log(LOG_CRIT, "already running: [pid %ld].",
+		       pid);
 		exit(LSB_EXIT_OK);
 	}
-
+	
 	if (apphbd_config.debug_level) {
 		if (apphbd_config.logfile[0] != EOS) {
 			cl_log_set_logfile(apphbd_config.logfile);
@@ -1042,6 +1059,7 @@ init_start()
 	drop_privs(0, 0); /* Become nobody */
 	g_main_run(mainloop);
 	return_to_orig_privs();
+	cl_log(LOG_DEBUG, "testing write to syslog when i am a nobody");
 	close_watchdog();
 	wconn->ops->destroy(wconn);
 	if (unlink(PIDFILE) == 0) {
@@ -1055,27 +1073,26 @@ make_daemon(void)
 {
 	int	j;
 	long	pid;
-	FILE *	lockfd;
 
+
+#ifndef NOFORK
 	pid = fork();
 
 	if (pid < 0) {
-		cl_log(LOG_CRIT, "cannot start daemon");
+		cl_log(LOG_CRIT, "cannot start daemon.");
 		exit(LSB_EXIT_GENERIC);
 	}else if (pid > 0) {
 		exit(LSB_EXIT_OK);
 	}
-
-	lockfd = fopen(PIDFILE, "w");
-	if (lockfd == NULL) {
-		cl_log(LOG_CRIT, "cannot create pid file" PIDFILE);
-		exit(LSB_EXIT_GENERIC);
-	}else{
-		pid = getpid();
-		fprintf(lockfd, "%ld\n", pid);
-		fclose(lockfd);
+#endif
+		
+	if (cl_lock_pidfile(PIDFILE) < 0) {
+		cl_log(LOG_CRIT, "already running: [pid %d].",
+		       cl_read_pidfile(PIDFILE));
+		exit(LSB_EXIT_OK);
 	}
 
+	
 	umask(022);
 	getsid(0);
 	if (!apphbd_config.debug_level) {
@@ -1091,37 +1108,13 @@ make_daemon(void)
 
 }
 
-static long
-get_running_pid(gboolean* anypidfile)
-{
-	long    pid;
-	FILE *  lockfd;
-	lockfd = fopen(PIDFILE, "r");
-
-	if (anypidfile) {
-		*anypidfile = (lockfd != NULL);
-	}
-
-	if (lockfd != NULL
-	&&      fscanf(lockfd, "%ld", &pid) == 1 && pid > 0) {
-		if (CL_PID_EXISTS((pid_t)pid)) {
-			fclose(lockfd);
-			return(pid);
-		}
-	}
-	if (lockfd != NULL) {
-		fclose(lockfd);
-	}
-	return(-1L);
-}
-
 static int
 init_stop(void)
 {
 	long	pid;
 	int	rc = LSB_EXIT_OK;
-	pid =	get_running_pid(NULL);
-
+	pid =	cl_read_pidfile(PIDFILE);
+	
 	if (pid > 0) {
 		if (CL_KILL((pid_t)pid, SIGTERM) < 0) {
 			rc = (errno == EPERM
@@ -1138,19 +1131,13 @@ init_stop(void)
 static int
 init_status(void)
 {
-	gboolean	anypidfile;
-	long	pid =	get_running_pid(&anypidfile);
+	long	pid =	cl_read_pidfile(PIDFILE);
 
 	if (pid > 0) {
 		fprintf(stderr, "%s is running [pid: %ld]\n"
 		,	cmdname, pid);
 		return LSB_STATUS_OK;
 	}
-	if (anypidfile) {
-		fprintf(stderr, "%s is stopped [pidfile exists]\n"
-		,	cmdname);
-		return LSB_STATUS_VAR_PID;
-	}
 	fprintf(stderr, "%s is stopped.\n", cmdname);
 	return LSB_STATUS_STOPPED;
 }
@@ -1222,7 +1209,7 @@ close_watchdog(void)
 		if (write(watchdogfd, "V", 1) != 1) {
 			cl_log(LOG_CRIT
 			,	"Watchdog write magic character failure:"
-			" closing watchdog!");
+			" closing watchdog!\n");
 		}
 		close(watchdogfd);
 		watchdogfd=-1;
@@ -1235,7 +1222,7 @@ tickle_watchdog(void)
 	if (watchdogfd >= 0) {
 		if (write(watchdogfd, "", 1) != 1) {
 			cl_log(LOG_CRIT
-			,	"Watchdog write failure: closing watchdog!");
+			,	"Watchdog write failure: closing watchdog!\n");
 			close_watchdog();
 			watchdogfd=-1;
 		}
@@ -1268,7 +1255,7 @@ load_notification_plugin(const char * pluginname)
 	PIL_rc	rc;
 	void*	exports;
 	if (pisys == NULL) {
-		pisys = NewPILPluginUniv("/usr/lib/heartbeat/plugins");
+		pisys = NewPILPluginUniv(HA_LIBHBDIR "/heartbeat/plugins");
 		if (pisys == NULL) {
 			return FALSE;
 		}
